  function[f,Jout] = sfdjac(fun,x,Extra,m,verb,JPI)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Estimate sparse Jacobian matrix J by sparse finite
% differences.
%
% INPUT:
%  fun : Handle to the function
%   x: Estimate J at the current point x.
%   J: the sparse matrix structure of the Jacobian
%   group: group(i) = j means that column i belongs to group (or color)
%          j. Each group (or color) corresponds to a finite difference.
%
% OUTPUT:
%   J: the sparse Jacobian estimate.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
n=length(x);
 
% Get the Sparsity Pattern of the Jacobian Matrix
if nargin < 6
 JPI=[];
end
if isempty(JPI)
JPI=getJPI(fun,m,n,Extra,'c');
end
J=JPI.SPJ;
group=JPI.group;
m=size(J,1);
if isempty(verb) verb=0; end
if (verb >=1)
        disp(sprintf('Number of Column groups = %d',max(group)));
end
if (verb >= 2)
        subplot(1,1,1)
        spy(J);
        title 'Sparsity Structure of J'
end

  ncol = max(group);
  V = zeros(n,ncol);
  JV = zeros(n,ncol);
 
  for k = 1:ncol
      V(:,k) = (group == k);
  end

  if ncol >0
        [f,JV] =forwprod(fun,x,V,m,Extra);
  else
	f=funcvalJ(fun,x,m,Extra);
   end
  [i, j]=find(J);
  group(find(group==0))=ones(length(find(group==0)),1);
  Jout = sparse(i,j,JV((group(j)-1)*m + i),m,n);

%  Jout=sparse(m,n); 
%  for k = 1:ncol
%      cols = find(V(:,k)); 
%      [j,i] = find(J(:,cols)');
%      if(length(i)>0)
%      	val = JV(i,k);
%      	Jout(:,cols) = sparse(i,j,val,m,length(cols));
%      end
%  end
