function [val,H]=evalH(fun,x,w,Extra,HPI,verb)
%
%evalJ  Compute Sparse Hessians Via AD
%       evalH is part of ADMIT toolbox. It computes sparse Hessian
%       of a general scalar nonlinear mapping.        
%
%       The user's function is identified by a unique identifier
%       fun. See ADMIT users manual for detail.
%
%       val=evalH(fun,x)  computes just the function value at point x. 
%       [val,grad]=evalH(fun,x) Also computes the n x 1 gradient at x.
%       [val,grad,H]=evalH(fun,x) Also computes the n x n
%                    sparse Hessian matrix at x.
%
%       Please refer to ADMIT users manual for complete reference. 
%       
%       HPI argument can be used to efficient execution. SEE GetHPI, dispHPI
%       Extra contains extra matrix argument for user's function. See users manual
%       for reference
%
%       ALSO SEE configF, evalJ
%
%
%       ******************************************************************
%       *              ADMIT Project   10-1-96                           *
%       *              Copyright (c) 1996 Cornell University.            *
%       ******************************************************************
 
  
  if nargin < 3 error('Need at least Three input Argument. see help evalHTens'); end
  if nargin > 7 error('Need at most seven input Arguments. see help evalHTens');
  end

  if nargin < 4 Extra=[]; end
  if nargin < 5 HPI = []; end
  if nargin < 6 verb = 0; end
  
  % MEX routine #1
	m=length(w);
	  val=funcvalJ(fun,x,m,Extra);

  % MEX routine #2
  if nargout == 2
		  H=HessRecoverTens(fun,x,w,Extra,verb,HPI);
		  % Make it numerically symmetric
		  H=(H+H')/2; 
  end

