function [f,extJ,ns,J]=evalJ(class,fun,x,fdata,EJPI,verb)
%
%evalJExt
%
%       Compute Extended Jacobian, Newton step. 
%
%       f=evalJExt(class,fun,x)  
%       [f,extJ]=evalJExt(class,fun,x) 
%       [f,extJ,ns]=evalJExt(class,fun,x) 
%       [f,extJ,ns,J]=evalJExt(class,fun,x) 
%
%       Please refer to ADMIT-2 users manual for complete reference.
%
%       EJPI argument can be used to efficient execution. 
%       SEE GetExtJPI
%
%       ALSO SEE configureExtfun, evalHExt, ExtJV, ExtWJ, ExtHV, ExtVHV
%
%
%       ******************************************************************
%       *              ADMIT-2 Project   1996-97                         *
%       *              Copyright (c) 1996 Cornell University.            *
%       ******************************************************************
 
n=length(x);

% Initialize

if nargin < 4 fdata = []; end
if nargin < 5 EJPI=[]; end
if nargin < 6 verb=0; end

if isempty(fdata) 
	[fun,fdata]=MakeExtFdata(class,fun,n); 
end

if nargout == 1
	f=funcvalJExt(class,fun,x,fdata);
else

if isempty(EJPI)
		EJPI=GetExtJPI(class,fun,n,fdata);
end
if class=='GF'
	[f,extJ]=evalJgen(fun,fdata,x,EJPI);
end
if class=='GC'
	if size(fun,1)==1
        [f,extJ]=evalJCF(fun,fdata,x,EJPI);
	else
        [f,extJ]=evalJGC(fun,fdata,x,EJPI);
	end
end
if class=='GP'
	if size(fun,1)==1
        [f,extJ]=evalJPS(fun,fdata,x,EJPI);
	else
        [f,extJ]=evalJGP(fun,fdata,x,EJPI);
	end
end
if class=='DO'
        [f,extJ]=evalJDO(fun,fdata,x,EJPI);
end
if class=='GI'
	impl=fdata.impl;
	if impl==0
        if fdata.kind==1
        [f,extJ]=evalJDI1(fun,fdata,x,EJPI);
        elseif fdata.kind==2
        [f,extJ]=evalJDI2(fun,fdata,x,EJPI);
        else
        [f,extJ]=evalJDI3(fun,fdata,x,EJPI);
        end
	else
	if fdata.kind==1
        [f,extJ]=evalJII1(fun,fdata,x,EJPI);
        elseif fdata.kind==2
        [f,extJ]=evalJII2(fun,fdata,x,EJPI);
        else
        [f,extJ]=evalJII3(fun,fdata,x,EJPI);
        end
	end
end


outm=length(f);
[nrows,ncols]=size(extJ);
if nargout == 3
	if nrows==outm
		if outm >= n
		ns=-extJ\f;
		else
		disp('Underdetermined system, newton step cannot be calculated');
	        end
	else
		[L,U,P]=lu(extJ);
        	rhs=[zeros(nrows-outm,1);-f];
        	if outm >= n
			ns=U\(L\(P*rhs));
        		ns=ns(1:n);
		else
			disp('Underdetermined system, newton step cannot be calculated');
		end
	end
end
 
if nargout==4
        B=extJ(nrows-outm+1:nrows,1:n);
	if nrows==outm
                J=B;
        else
                A=extJ(1:nrows-outm,1:n);
                L=extJ(1:nrows-outm,n+1:ncols);
                M=extJ(nrows-outm+1:nrows,n+1:ncols);
                J=B-M*(L\A);
        end
	ns=-J\f;
end
 

 
        if verb >= 1
		if class=='GP'
                disp(sprintf('Number of Partially separable components = %d',fdata(1)));
		end
		if class=='GC'
                disp(sprintf('Number of composite functions = %d',fdata(1)));
		end
		if class=='GF'
                disp(sprintf('Number of functions in general structured program = %d',fdata(1)));
		end
		if class=='GI'
                disp(sprintf('The time interval =[%f,%f], step size = %f',fdata(1),fdata(2),fdata(3)));
		end
                disp(sprintf('Problem Size = %d -> %d',n,outm));
        end
        if verb==2
                if nargout==4
                        subplot(2,1,1)
                        spy(extJ)
                        title 'Sparsity Structure of Extended Jacobian Matrix'
                        subplot(2,1,2)
                        spy(J)
                        title 'Sparsity Structure of True Jacobian Matrix'
                else
                        spy(extJ)
                        title 'Sparsity Structure of Extended Jacobian Matrix'
                end
        end
end
