function [PIout,SPH]= GetTPI(fun, n,m,Extra, method, SPH)
%
%GetTPI
%
%       Compute sparsity and coloring information for sparse Hessians.
%       This is really useful for efficiency of the  computing sparse
%       Hessians with evalH, because once computed sparsity and coloring
%       Information can be saved once for all.
%
%       The user's function is identified by a unique identifier
%       fun. See ADMIT users manual for detail.
%
%       PartInfo = GetTPI(fun,n)  computes  sparsity + coloring Info for nxn H.
%
%       Please refer to ADMIT users manual for complete reference.
%
%       Extra contains extra matrix argument for user's function. See users manual
%       for reference
%
%       ALSO SEE evalH, dispHPI
%
%
%       ******************************************************************
%       *              ADMIT Project   10-1-96                           *
%       *              Copyright (c) 1996 Cornell University.            *
%       ******************************************************************

if (nargin < 2) error('need at least 2 input arguments'); end
if nargin < 3 m=[]; end
if nargin < 4 Extra=[]; end
if isempty(m) m=n; end
if nargin < 5 method ='i-a'; end
if (nargin < 6) SPH=[]; end
 
if isempty(SPH)
SPH=tenssp(fun,n,m,Extra);
end
SPH=spones(SPH);
 
if (method=='i-a')|(method=='i-f')
        H2=spones(SPH*SPH);
        H2=spones(SPH*SPH+speye(n));
        p=id(H2,full(sum(H2)));
        g=colorHess(H2,p);
        PartInfo=sparse(n+1,n);
        PartInfo=[SPH; g'];
 
elseif (method=='d-a')|(method=='d-f')
        [g,o]=dirhess(SPH);
        PartInfo=sparse(n+2,n);
        PartInfo=[SPH; g';o'];
else
        o=id(SPH,full(sum(SPH)));
        colors=zeros(n,1);
        J=tril(SPH(o,o));
        H1=spones(J'*J);
        p1=id(H1,full(sum(H1)));
        colors(o)=colorHess(H1,p1);
        PartInfo=sparse(n+2,n);
        PartInfo=[SPH; colors';o'];
 
end

if method=='i-a'
        mval=1;
end
if method=='d-a'
        mval=2;
end
if method=='s-a'
        mval=3;
end
if method=='i-f'
        mval=4;
end
if method=='d-f'
        mval=5;
end
if method=='s-f'
        mval=6;
end
 
[ms,ns] = size(PartInfo);
[si,sj, v] = find(PartInfo);
PIout = [mval;length(si);ms; ns; si; sj; v];
