function argout=guioption(action,arg)
%
% ADMIT-2 Graphical User Interface
%
global MODEL
global SO
global X
global R
global Q
global T
global SIG
global CALLV
global FUN
global sigmaguess
global Rguess
global DT
global ydata

global call
if nargin < 1
	action='initialize';
end

fig=findobj(get(0,'Children'),'Tag','OPTIONSGUI');
fig2=findobj(get(0,'Children'),'Tag','SETUPWIN');

if strcmp(action,'initialize')==1
%initialize
MODEL='bls';
SO=100;
X=95;
R=.1;
Q=0;
T=0.25;
DT=0.05;
SIG=0.5;
CALLV=10;
FUN='sqrmin';
sigmaguess=0.3;
Rguess = 0.1;
	if isempty(fig)
		fig = figure('NumberTitle','off');
		set(fig,'Tag','OPTIONSGUI');
	 model=uimenu(fig,'Label','Model');
                set(model,'Tag','MODEL');

	setup=uimenu(fig,'Label','Setup Parameters','CallBack',['guifintoolbox(''setupfun'')']);
		set(fig,'Tag','OPTIONSGUIOPT');
	setupopt=uimenu(fig,'Label','Optimization parms','CallBack',['guifintoolbox(''setupfunopt'')']);
		set(setupopt,'Tag','SETUPOPT');
		method=uimenu(fig,'Label','Methods');
		set(method,'Tag','METHOD');
		misc=uimenu(fig,'Label','View');
		set(misc,'Tag','VIEW');
	end
model_bls=uimenu(model,'Label','Black-Scholes pricing model','Separator','on','CallBack','check(1)');
model_bin=uimenu(model,'Label','Binomial pricing model','Separator','on','CallBack','check(2)');
set(model_bls,'Tag','model_bls');
set(model_bin,'Tag','model_bin');
set(model_bls,'checked','on');
set(model_bin,'checked','off');

method_solve=uimenu(method,'Label','solve','CallBack','guifintoolbox(''solve'');');
method_imp=uimenu(method,'Label','Implied volatility','CallBack','guifintoolbox(''imp'');');
method_datafit=uimenu(method,'Label','Datafit','CallBack','guifintoolbox(''datafit'');');
method_minimize=uimenu(method,'Label','Minimize','CallBack','guifintoolbox(''minimize'');');


misc_view=uimenu(misc,'Label','View final plot','CallBack','guifintoolbox(''viewres'');');
enable3d=uimenu(misc,'Label','enable-3d-rot','Separator','on','CallBack','guifintoolbox(''enable3d'')');

elseif strcmp(action,'viewres')==1
[m,n]=size(call);
if (m > 1) & (n> 1)
surf(T,SO,call);
xlabel 'Time'
ylabel 'Asset Price'
zlabel 'Option Value'
else
if length(call) > 1
plot(SO,call);
xlabel 'Asset Price'
ylabel 'Option Value'
end
end
elseif strcmp(action,'enable3d')==1
rotate3d;

elseif strcmp(action,'solve')==1
call=[];
if MODEL=='bls'
if (length(SO) > 1) & (length(T) > 1)
if (length(SIG) == 1) 
for j=1:length(SO)
call(:,j) = blsprice(SO(j),X,R,T,SIG,Q);
end
else
if ~isequal(length(SIG),length(SO))
error('Length of volatility vector should be same as the asset price, otherwise it should be a scalar'); end
for j=1:length(SO)
call(:,j) = blsprice(SO(j),X,R,T,SIG(j),Q);
end
end
else
call = blsprice(SO,X,R,T,SIG,Q);
end
if length(call)==1
call
end
else
for j=1:length(SO)
for i=1:length(T)
if length(SIG)==1
[p,o] = binprice(SO(j),X,R,T(i),DT,SIG,1,Q);
else
if ~isequal(length(SIG),length(SO))
error('Length of volatility vector should be same as the asset price, otherwise it should be a scalar'); end
[p,o] = binprice(SO(j),X,R,T(i),DT,SIG(j),1,Q);
end
call(i,j)=o(1,1);
end
end
if length(call)==1
call
end
end
elseif strcmp(action,'imp')==1

if (length(SO) > 1) & (length(T) > 1)
for j=1:length(SO)
sigma(:,j) = blsimpv(SO(j),X,R,T,CALLV);
end
else
sigma = blsimpv(SO,X,R,T,CALLV);
end

sigma

elseif strcmp(action,'datafit')==1
if MODEL=='bls'
ydata.sigmaguess=sigmaguess;
ydata.Rguess=Rguess;
ydata.SIG=SIG;
ydata.SO=SO
ydata.X=X;
ydata.R=R;
ydata.T=T;
ydata.DT=DT;
ydata.Q=Q;
ydata.CALLV=CALLV
ydata.fun=FUN;
x=solvebls(ydata);
%x=snls(ADfun('blsfun'),sigmaguess,[],[],ydata);
else
ydata.sigmaguess=sigmaguess;
ydata.SO=SO;
ydata.X=X;
ydata.R=R;
ydata.T=T;
ydata.DT=DT;
ydata.Q=Q;
ydata.CALLV=CALLV;
ydata.fun=FUN;
x=solvebin(ydata);
%x=snls(ADfun('binfun'),sigmaguess,[],[],ydata);
end

x

elseif strcmp(action,'minimize')==1
ydata.sigmaguess=sigmaguess;
ydata.SO=SO;
ydata.X=X;
ydata.R=R;
ydata.T=T;
ydata.DT=DT;
ydata.Q=Q;
ydata.CALLV=CALLV;
ydata.fun=FUN;
if MODEL=='bls'
x=solveblsmin(ydata);
%x=sfmin(ADfun('blsfunmin',1),sigmaguess,[],[],ydata);
else
x=solvebinmin(ydata);
%x=sfmin(ADfun('binfunmin',1),sigmaguess,[],[],ydata);
end
x

elseif strcmp(action,'setupfunopt')==1
if ~isempty(fig2) close(fig2); end
fig2 = figure('NumberTitle','off');
set(fig2,'Tag','SETUPWINOPT');
        GUIfinsetupopt(fig2);

elseif strcmp(action,'setupfun')==1
if ~isempty(fig2) close(fig2); end
fig2 = figure('NumberTitle','off');
set(fig2,'Tag','SETUPWIN');
	GUIfinsetup(fig2);
end
