  function[f,Jout] = sfdjac(fun,x,Extra,m,verb,JPI)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% INPUT:
%  fun: Integer Handle for the function.
%   x: Estimate J at the current point x.
%   J: the sparse matrix structure of the Jacobian
%   group: group(i) = j means that column i belongs to group (or color)
%          j. Each group (or color) corresponds to a finite difference.
%
% OUTPUT:
%   J: the sparse Jacobian estimate.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% get the Sparsity Pattern of the Jacobian Matrix
n=length(x);
if nargin < 6
JPI=[];
end

if isempty(JPI)
JPI=getJPI(fun,m,n,Extra,'d');
end
	index=2;
	len = JPI(index);
        si = JPI(index+3:index+2+len);
        sj = JPI(index+3+len:index+2+2*len);
        v = JPI(index+3+2*len:index+2+3*len);
	PartInfo = sparse(si,sj,v,JPI(index+1),JPI(index+2),len);

m=PartInfo(size(PartInfo,1),n+1);
SPJr=PartInfo(1:m,n+1:2*n);
SPJc=PartInfo(1:m,1:n);
gr=PartInfo(1:m,2*n+1);
gc=PartInfo(m+1,1:n);
gc=gc';
gr=full(gr);
gc=full(gc);
nrow=size(PartInfo,2)-1-2*n;
W=PartInfo(1:m,2*n+2:2*n+1+nrow);
W=full(W);
ncol=size(PartInfo,1)-1-m;
V=PartInfo(m+2:m+1+ncol,1:n);
V=V';
V=full(V);


if isempty(verb) verb=0; end
if (verb >=1)
        disp(sprintf('Number of Row groups = %d',max(gr)));
        disp(sprintf('Number of column groups = %d',max(gc)));
        disp(sprintf('Total Number of groups = %d',max(gc)+max(gr)));
end
if (verb >= 2)
        subplot(2,2,1);
        spy(SPJr+SPJc);
        title 'Sparsity Structure of J'
        subplot(2,2,2);
        spy(SPJr);
        title 'Jr: Part computed by reverse AD'
        subplot(2,2,3);
        spy(SPJc);
        title 'Jc: Part computed by forward AD'
end

Jr=sparse(m,n);
Jc=sparse(m,n);

if (ncol > 0)
% Call the ADOLC Driver to do forwardMode product
[f,JV]=forwprod(fun,x,V,m,Extra);
% Identify Elements
  [i, j]=find(SPJc);
  gc(find(gc==0))=ones(length(find(gc==0)),1);
  Jc = sparse(i,j,JV((gc(j)-1)*m + i),m,n);
end



% Form the Matrix W, based on the coloring.
% W is a 0-1 Matrix
if (nrow > 0)
% Call the ADOLC Driver to do ReverseMode product
[f,WtJ] =revprod(fun,x,W,Extra);
 
% Identify Elements
  gr(find(gr==0))=ones(length(find(gr==0)),1);
  [i, j]=find(SPJr);
  Jr = sparse(i,j,WtJ((gr(i)-1)*n + j),m,n);
end
if isempty(f) f=funcvalJ(fun,x,m,Extra); end
Jout=Jr+Jc;
