  function[f,Jout] = sfdjac(fun,x,Extra,m,verb,JPI)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Estimate sparse Jacobian matrix J by sparse finite
% differences.
%
% INPUT:
%  fun : Handle to the function
%   x: Estimate J at the current point x.
%   J: the sparse matrix structure of the Jacobian
%   group: group(i) = j means that column i belongs to group (or color)
%          j. Each group (or color) corresponds to a finite difference.
%
% OUTPUT:
%   J: the sparse Jacobian estimate.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
n=length(x);
 
if nargin < 6
JPI=[];
end
 
if isempty(JPI)
JPI=getJPI(fun,m,n,Extra,'r');
end
        index=2;
        len = JPI(index);
        si = JPI(index+3:index+2+len);
        sj = JPI(index+3+len:index+2+2*len);
        v = JPI(index+3+2*len:index+2+3*len);
PartInfo = sparse(si,sj,v,JPI(index+1),JPI(index+2),len);
 

[m,xx]=size(PartInfo);
J=PartInfo(:,1:n);
group=PartInfo(:,n+1);
group=full(group);
if isempty(verb) verb=0; end
if (verb >=1)
        disp(sprintf('Number of Row groups = %d',max(group)));
end
if (verb >= 2)
        subplot(1,1,1)
        spy(J);
        title 'Sparsity Structure of J'
end

  ncol = max(group);
  W = zeros(m,ncol);
  for k = 1:ncol
      W(:,k) = (group == k);
  end
 

  if ncol > 0
[f,WtJ] =revprod(fun,x,W,Extra);
  else
        f=funcvalJ(fun,x,m,Extra);
   end
 
  [i, j]=find(J);
  group(find(group==0))=ones(length(find(group==0)),1);
  Jout = sparse(i,j,WtJ((group(i)-1)*n + j),m,n);

%  Jout=sparse(m,n); 
%  for k = 1:ncol
%      rows = find(W(:,k));
%      [i,j] = find(J(rows,:));
%      if (length(i) >0)
%        val=WtJ(j,k);
%        Jout(rows,:)= sparse(i,j,val,length(rows),n);
%      end
%  end
 

