  function[f,Jout] = sfdjac(fun,x,Extra,m,verb,JPI,stepsize)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Estimate sparse Jacobian matrix J by sparse finite
% differences.
%
% INPUT:
%  fun : Handle to the function
%   x: Estimate J at the current point x.
%   J: the sparse matrix structure of the Jacobian
%   group: group(i) = j means that column i belongs to group (or color)
%          j. Each group (or color) corresponds to a finite difference.
%
% OUTPUT:
%   J: the sparse Jacobian estimate.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
n=length(x);
 
if nargin < 6
JPI=[];
end

if isempty(JPI)
% Get the Sparsity Pattern of the Jacobian Matrix
JPI= getJPI(fun, m, n,Extra, 'f');
end
        index=2;
        len = JPI(index);
        si = JPI(index+3:index+2+len);
        sj = JPI(index+3+len:index+2+2*len);
        v = JPI(index+3+2*len:index+2+3*len);
        PartInfo = sparse(si,sj,v,JPI(index+1),JPI(index+2),len);
 
[m,n]=size(PartInfo);
m=m-1;
J=PartInfo(1:m,:);
group=PartInfo(m+1,:);
group=group';
group=full(group);
if isempty(verb) verb=0; end
if (verb >=1)
        disp(sprintf('Number of SFD groups = %d',max(group)));
end
if (verb >= 2)
        subplot(1,1,1)
        spy(J);
        title 'Sparsity Structure of J'
end

  [m,n] = size(J); 
  ncol = max(group); 
  for k = 1:ncol
      V(:,k) = (group == k);
  end
  epsvec=stepsize.*ones(ncol,1);;
  if ncol > 0
	  [f,v] =fdapprox(fun,x,V,m,epsvec,Extra);
  else
  	f=funcvalJ(fun,x,m,Extra);
  end
  [i, j]=find(J);
  group(find(group==0))=ones(length(find(group==0)),1);
  Jout = sparse(i,j,v((group(j)-1)*m + i),m,n);

