function [numg]=dispJPI(JPI,n,method,verb)
%
%dispJPI
%
%       Display sparsity and coloring information for sparse Jacobians
%       computed by GetJPI.
%
%       The level of output can be controlled by parameter verb.
%       verb = 0  => No display
%       verb = 1  => Number of groups are shown.
%       verb = 2  => The sparsity pattern of Hessian is also shown
%
%       dispJPI(PartInfo,1) 
%
%       Please refer to ADMIT users manual for complete reference.
%
%       ALSO SEE evalJ, GetJPI
%
%
%       ******************************************************************
%       *              ADMIT Project   10-1-96                           *
%       *              Copyright (c) 1996 Cornell University.            *
%       ******************************************************************

if nargin < 3 method ='d'; end
if nargin < 4 verb=1; end
if isempty(method) method ='d'; end
if isempty(verb) verb=1; end

if method ~='s'
        index=2;
        len = JPI(index);
        si = JPI(index+3:index+2+len);
        sj = JPI(index+3+len:index+2+2*len);
        v = JPI(index+3+2*len:index+2+3*len);
        PartInfo = sparse(si,sj,v,JPI(index+1),JPI(index+2),len);
else
	PartInfo1=JPI(2:length(JPI));
	[PartInfo,A,p1,p,q]=ExtractSubJPI(PartInfo1);
end

if (method=='s')
m=PartInfo(size(PartInfo,1),n+1);
Jc=PartInfo(1:m,1:n);
Jr=PartInfo(1:m,n+1:2*n);
nrow=size(PartInfo,2)-5-2*n;
ncol=size(PartInfo,1)-3-m;
numg=nrow+ncol; 
if (verb >=1)
        disp(sprintf('Size of the Jacobian = %dx%d',m,n));
        disp(sprintf('Number of Row groups = %d',nrow));
        disp(sprintf('Number of column groups = %d',ncol));
        disp(sprintf('Total Number of groups = %d',nrow+ncol));
end
if (verb >= 2)
        subplot(2,2,1);
        spy(Jr+Jc);
        title 'Sparsity Structure of J'
        subplot(2,2,2);
        spy(Jr);
        title 'Jr: Part computed by reverse AD'
        subplot(2,2,3);
        spy(Jc);
        title 'Jc: Part computed by forward AD'
end

elseif (method =='d')	
m=PartInfo(size(PartInfo,1),n+1);
Jr=PartInfo(1:m,n+1:2*n);
Jc=PartInfo(1:m,1:n);
nrow=size(PartInfo,2)-1-2*n;
ncol=size(PartInfo,1)-1-m;
numg=nrow+ncol; 
if (verb >=1)
        disp(sprintf('Size of the Jacobian = %dx%d',m,n));
        disp(sprintf('Number of Row groups = %d',nrow));
        disp(sprintf('Number of column groups = %d',ncol));
        disp(sprintf('Total Number of groups = %d',nrow+ncol));
end
if (verb >= 2)
        subplot(2,2,1);
        spy(Jr+Jc);
        title 'Sparsity Structure of J'
        subplot(2,2,2);
        spy(Jr);
        title 'Jr: Part computed by reverse AD'
        subplot(2,2,3);
        spy(Jc);
        title 'Jc: Part computed by forward AD'
end

elseif (method=='c')|(method=='f')

[m,n]=size(PartInfo);
m=m-1;
J=PartInfo(1:m,:);
group=PartInfo(m+1,:);
group=group';
group=full(group);
numg=max(group); 
if (verb >=1)
        disp(sprintf('Size of the Jacobian = %dx%d',m,n));
        disp(sprintf('Number of Column groups = %d',max(group)));
end
if (verb >= 2)
        subplot(1,1,1)
        spy(J);
        title 'Sparsity Structure of J'
end

else
[m,n]=size(PartInfo);
n=n-1;
J=PartInfo(1:m,:);
group=PartInfo(:,n+1);
group=full(group);
numg=max(group);
if (verb >=1)
        disp(sprintf('Size of the Jacobian = %dx%d',m,n));
        disp(sprintf('Number of Column groups = %d',max(group)));
end
if (verb >= 2)
        subplot(1,1,1)
        spy(J);
        title 'Sparsity Structure of J'
end

end
