function [f,JV] = forwprod(fun,x,V,m,Extra,epsvec)
%
%forwprod
%
%	Computes forward mode product J*V using ADOLC 
%
%       The user's function is identified by a unique identifier
%       fun. See ADMIT users manual for detail.
%
%       [f,JV]=forwprod(fun,x,V) computes J*V at point x.
%
%       Please refer to ADMIT users manual for complete reference.
%
%       ALSO SEE revprod, funcvalJ, JacSP
%
%
%       ******************************************************************
%       *              ADMIT Project   10-1-96                           *
%       *              Copyright (c) 1996 Cornell University.            *
%       ******************************************************************

n=length(x); 
if nargin < 5 Extra=[]; end

if nargin < 4 m=[]; end
if isempty(m) m=n; end
if ~isstr(fun)

	if isempty(Extra)
		[f,JV]=maindriver(1,fun,x,V,m);
	else
		[f,JV]=maindriver(1,fun,x,V,m,Extra);
	end

else
	epsvec=1e-5.*ones(size(V,2),1); 
        fun=deblank(fun);
        f=feval(fun,x,m,Extra);
	JV=zeros(m,size(V,2));
	for i=1:size(V,2)
                val=feval(fun,x+epsvec(i).*V(:,i),m,Extra);
                JV(:,i)=(val-f)./epsvec(i);
        end
end

