function TV = Tensprod(fun,x,V,w,Extra,stepsize)
%
%Tensprod
%
%       Computes the Hessian Matrix product H*V, where
%       H is the Hessian of (F(x)'*w) (a tensor vector product)
%
%       The user's function is identified by a unique identifier
%       fun. See ADMIT users manual for detail.
%
%
%       TV=Tensprod(fun,x,V,w)
%
%       Please refer to ADMIT users manual for complete reference.
%
%       ALSO SEE funcval, HessSP, HtimesV
%
%
%       ******************************************************************
%       *              ADMIT Project   10-1-96                           *
%       *              Copyright (c) 1996 Cornell University.            *
%       ******************************************************************
if nargin < 5 Extra=[]; end

if ~isstr(fun)

if isempty(Extra)
TV=maindriver(12,fun,x,V,w);
else
TV=maindriver(12,fun,x,V,w,Extra);
end

else
	p=size(V,2);
	fun=deblank(fun);
	n=length(x);
	m=length(w);

	f=feval(fun,x,m,Extra);		
	val=f'*w;
	grad=ones(n,1);
	for j=1:n
		y=x;
		y(j)=y(j)+stepsize;
		f=feval(fun,y,m,Extra);     
      	 	val2=f'*w;
		grad(j)=(val2-val)/stepsize;
	end	
	for k=1:p
		xx=x+stepsize*V(:,k);
		f=feval(fun,xx,m,Extra);     
       		val=f'*w;
		grad2=ones(n,1);
        
		for j=1:n       
               		 y=xx;
                	y(j)=y(j)+stepsize;
                	f=feval(fun,y,m,Extra);        
                	val2=f'*w;      
                	grad2(j)=(val2-val)/stepsize;  
        	end
		TV(:,k)=(grad2-grad)./stepsize;
	end	
	
end
