/*
   ----------------------------------------------------------------------------
   Author  : Uwe Geitner
   Hardware: IBM-compatible PC 386/40
   Software: g++-GNU-Compiler (v2.4) under Linux
   Date    : 14/07/95
   Content : - Definition of the methods for the class myBitPattern
             - derived from this class are class RowBitPattern and class
               ColBitPattern
               myBitPattern::OR(unsigned int, unsigned int, unsigned int),
               myBitPattern::OR_EQ(unsigned int, unsigned int),
               myBitPattern::OR_EQ(unsigned int, myBitPattern*, unsigned int),
               myBitPattern::AND(unsigned int, myBitPattern*, unsigned int),
               myBitPattern::AND(unsigned int, unsigned int),
               myBitPattern::AND_EQ(unsigned int, unsigned int),
               myBitPattern::EQUAL(unsigned int, myBitPattern*, unsigned int),
               myBitPattern::EQUAL(unsigned int, unsigned int),
               myBitPattern::setBit(unsigned int, unsigned int),
               myBitPattern::readBit(unsigned int, unsigned int),
               myBitPattern::showBitPattern(unsigned int),

ADDED by Arun Verma

    void TRANS     (const myBitPattern*, unsigned int) ;
    void ADD     (unsigned int, unsigned int, unsigned int,const myBitPattern*) ;
    void ADD     (unsigned int, unsigned int,const myBitPattern*) ;
    void ADD_EQ     (unsigned int, unsigned int,const myBitPattern*) ;
   ----------------------------------------------------------------------------
*/

#define _BITFELD_CPP

#include <memory.h>
#include <stdio.h>
#include <iostream.h>
#include "bitfeld.h"
#include "error.h"

myBitPattern::myBitPattern()
{
}

myBitPattern::~myBitPattern(void)
{
}

void myBitPattern::OR(unsigned int i, unsigned int j, unsigned int k)
{
  if ( (i >= arr_size) || (j >= arr_size) || (k >= arr_size) )
    error_report(1) ;
  else 
    for (int index=0; index<blocks; index++)
      bits[i][index] = bits[j][index] | bits[k][index] ;
}

void myBitPattern::TRANS(const myBitPattern* name,unsigned int i)
{

    for (int row=0; row<pat_size; row++)
    for (int col=0; col<blocks; col++)
	{ 
int loc=row*blocks+col;
bits[row][col]=name->getNumber(i,loc);
        }

}


void myBitPattern::ADD(unsigned int i, unsigned int j, unsigned int k,const myBitPattern* name )
{
  if ( (i >= arr_size) || (j >= arr_size) || (k >= arr_size) )
    error_report(1) ;
  else
	{
    for (int index=0; index<blocks; index++)
      bits[i][index] = bits[j][index] | bits[k][index] ;

    for (int row=0; row<name->getPatSize(); row++)
    for (int col=0; col<name->getBlocks(); col++)
	{ 
	int loc=row*name->getBlocks()+col;
        if (name->readBit(j,row)) 
	bits[i][loc] |= name->getNumber(k,col);
       if (name->readBit(k,row)) 
	bits[i][loc] |= name->getNumber(j,col);
	}
}
}

void myBitPattern::ADD(unsigned int i, unsigned int j,const myBitPattern* name )
{
  if ( (i >= arr_size) || (j >= arr_size) )
    error_report(1) ;
  else
    for (int row=0; row<name->getPatSize(); row++)
    for (int col=0; col<name->getBlocks(); col++)
	{ 
	int loc=row*name->getBlocks()+col;
      if (name->readBit(j,row))
	bits[i][loc] |= name->getNumber(j,col);
	}
}

void myBitPattern::ADD_EQ(unsigned int i, unsigned int j,const myBitPattern* name )
{
  if ( (i >= arr_size) || (j >= arr_size) )
    error_report(1) ;
  else
	{
    for (int index=0; index<blocks; index++)
      bits[i][index]|= bits[j][index] ;
    for (int row=0; row<name->getPatSize(); row++)
    for (int col=0; col<name->getBlocks(); col++)
	{ 
	int loc=row*name->getBlocks()+col;
      if (name->readBit(i,row))
	bits[i][loc] |= name->getNumber(j,col);
      if (name->readBit(j,row))
	bits[i][loc] |= name->getNumber(i,col);
	}
	}
}

void myBitPattern::ADD_EQ(unsigned int i, unsigned int j,unsigned int k, const myBitPattern* name )
{
  if ( (i >= arr_size) || (j >= arr_size) )
    error_report(1) ;
  else
        {
    for (int index=0; index<blocks; index++)
      bits[i][index]|= bits[j][index]|bits[k][index] ;
    for (int row=0; row<name->getPatSize(); row++)
    for (int col=0; col<name->getBlocks(); col++)
        {
        int loc=row*name->getBlocks()+col;
      if (name->readBit(j,row))
        bits[i][loc] |= name->getNumber(k,col);
      if (name->readBit(k,row))
        bits[i][loc] |= name->getNumber(j,col);
        }
        }
}
 

void myBitPattern::OR_EQ(unsigned int i, unsigned int j)
{
  if ( (i >= arr_size) || (j >= arr_size) )
    error_report(2) ;
  else 
    for (int index=0; index<blocks; index++)
      bits[i][index] |= bits[j][index] ;
}

void myBitPattern::OR_EQ(unsigned int row,  
                         const myBitPattern* name,
                         unsigned int other_row)
{
  if ( (pat_size != name->getPatSize()) || (row >= arr_size) || 
       (other_row>= name->getArrSize()) )
    error_report(3)  ;
  else
  {
    for (int index=0; index<blocks; index++)
      bits[row][index] |= name->getNumber(other_row,index) ;
  }
}

flag2 myBitPattern::AND(unsigned int row, 
                       const myBitPattern* name, 
                       unsigned int other_row)
{
  if ( (pat_size != name->getPatSize()) || (row >= arr_size) || 
       (other_row>= name->getArrSize()) )
  {
    error_report(4) ;
    return falze ;
  }
  else
  {
    for (int index=0; index<blocks; index++)
      if (bits[row][index] & name->getNumber(other_row,index))  return truu ;
    return falze ;
  }
}

flag2 myBitPattern::AND(unsigned int i, unsigned int j)
{
  if ( (i >= arr_size) || (j >= arr_size) )
  {
    error_report(5) ;
    return falze ;
  } 
  else 
  {
    for (int index=0; index<blocks; index++)
      if (bits[i][index] & bits[j][index])  return truu ;
    return falze ;
  }
}

void myBitPattern::AND_EQ(unsigned int i, unsigned int j)
{
  if ( (i >= arr_size) || (j >= arr_size) )
    error_report(6) ;
  else 
    for (int index=0; index<blocks; index++)
      bits[i][index] &= bits[j][index] ;
}

void myBitPattern::EQUAL(unsigned int row,
                         const myBitPattern* name,
                         unsigned int other_row)
{
  if ( (pat_size != name->getPatSize()) || (row >= arr_size) || 
       (other_row>= name->getArrSize()) )
    error_report(7) ;
  else
    for (int i=0; i<blocks; i++)
      bits[row][i] = name->getNumber(other_row,i) ;
}

void myBitPattern::EQUAL(unsigned int row, unsigned int other_row)
{
  if ( (row >= arr_size) || (other_row>=arr_size) ) 
    error_report(8) ;   
  else
    for (int i=0; i<blocks;i++)
      bits[row][i] = getNumber(other_row,i) ;
}

void myBitPattern::setBit(unsigned int row, unsigned int pos)
{
  if ( (row >= arr_size) || (pos >= pat_size) )
    error_report(9) ;
  else 
    bits[row][pos/(8*sizeof(int))] |= (1<<(pos & (8*sizeof(int)-1))) ;
}

flag2 myBitPattern::readBit(unsigned int row, unsigned int pos) const
{
  int   nr ;
  if ( (row >= arr_size) || (pos >= pat_size) )
  {
    error_report(10) ;
    return falze ;
  }
  else
  {
    nr = bits[row][pos/(8*sizeof(int))] & (1 << (pos & (8*sizeof(int)-1))) ;
    if (nr)  return truu ;                     
    else     return falze ;
  }
}

void myBitPattern::showBitPattern(unsigned int number) const 
{
  int    help ;
  int    size_now ;

  for (int i=0; i<arr_size; i++)
  {
    if (number == 1)  cout << "row[" << i << "]\t  " ;
    else              cout << "col[" << i << "]\t  " ; 
    size_now = 0 ;
    for (int j=0; j<blocks; j++)
    {
      help = bits[i][j] ;
      for (int k=0; k<8*sizeof(int); k++)
      {
        if ((size_now++) >= pat_size)  break ;
        else if (help & (1 << k)) cout << 1 ;
             else                 cout << 0 ;
      } ;
      cout << " " ;
    }
    cout << "\n" ;
  }
  cout << "\n" ;
}

#undef _BITFELD_CPP


