/*
   ----------------------------------------------------------------------------
   Author  : Uwe Geitner
   Hardware: IBM-compatible PC 386/40
   Software: g++-GNU-Compiler (v2.4) under Linux
   Date    : 14/07/95
   Content : - Implementation of the methods and friend functions for the
               class ColBitPattern (base class is class mybitPattern) :
               ColBitPattern::ColBitPattern(),
               ColBitPattern::ColBitpattern(unsigned int, unsigned int),
               ColBitPattern::~ColBitPattern(void),
               ColBitPattern::estimateDegree(void),
               friend ColBitPattern* allocColBP(unsigned int, unsigned int)
   ----------------------------------------------------------------------------
*/

#define _COLBITPAT_CPP
#include <memory.h>
#include <stdio.h>
#include "bitfeld.h"
#include "colBitPat.h"
#include "MatLab.h"

ColBitPattern::ColBitPattern()
{
  arr_size = 0 ;
  pat_size = 0 ;
  blocks   = 0 ;
  NNZinMat = 0 ;
  col_nnz  = NULL ;
  degree   = NULL ;
  nachb_liste = NULL ;
  bits     = NULL ;
}

ColBitPattern::ColBitPattern(unsigned int rows, unsigned int psize)
{
  arr_size = rows ;
  pat_size = psize ;
  blocks   = (pat_size + 8*sizeof(int)-1) / (8*sizeof(int)) ;
  NNZinMat = 0 ;
  col_nnz  = new int[arr_size] ;
  memset(col_nnz, 0, arr_size * sizeof(int)) ;
  degree = new int[arr_size] ;
  memset(degree, 0, arr_size * sizeof(int)) ;
  nachb_liste = new Liste[arr_size] ;
  bits = new int*[arr_size] ;
  for (int i=0; i<arr_size; i++)
  /*  for (int j=0; j<blocks; j++) */
    {
      bits[i] = new int[blocks] ;
      memset(bits[i], 0, blocks * sizeof(int)) ;
    }
}

ColBitPattern::~ColBitPattern(void)
{
  if (col_nnz != NULL) delete[] col_nnz ;
  if (degree  != NULL) delete[] degree ;
  if (nachb_liste != NULL) delete[] nachb_liste ;
  if (bits != NULL)
  {
    for (int i=0; i<arr_size; i++)
      if (bits[i] != NULL) delete[] bits[i] ;
    delete[] bits ;
  }
}

void ColBitPattern::estimateDegree(void)
{
  int i, j;
  for (i=0; i<arr_size; i++)
    for (j=i+1; j<arr_size; j++)
      if (AND(i,j))
      {
        incDegree(i) ;
        addListElem(i,j) ;
        incDegree(j) ;
        addListElem(j,i) ;
      }
}

ColBitPattern* allocColBP(unsigned int rows, unsigned int psize)
{
  ColBitPattern*  help = new ColBitPattern ;
  help->arr_size = rows ;
  help->pat_size = psize ;
  help->blocks   = (help->pat_size + 8*sizeof(int)-1) / (8*sizeof(int)) ;
  help->NNZinMat = help->NNZinMat ;
  help->col_nnz  = new int[help->arr_size] ;
  memset(help->col_nnz, 0, help->arr_size * sizeof(int)) ;
  help->degree = new int[help->arr_size] ;
  memset(help->degree , 0, help->arr_size * sizeof(int)) ;
  help->nachb_liste = new Liste[help->arr_size] ;
  help->bits = new int*[help->arr_size] ;
  for (int i=0; i<help->arr_size; i++)
  {
    help->bits[i] = new int[help->blocks] ;
    memset(help->bits[i], 0, help->blocks * sizeof(int)) ;
  }
  return help ;
}

MatLab_output* ColBitPattern::MatLab(void) const
{
  MatLab_output*  help = alloc4MatLab(arr_size, NNZinMat) ;
  unsigned int counter = 0 ;
  
  for (int i=0; i<arr_size; i++)
    for (int j=0; j<pat_size; j++)
      if ( readBit(i,j) )   help->row_indices[counter++] = j ;      

  help->elem_cnt[0] = 0 ;
  for (int i=0; i<arr_size; i++)
    help->elem_cnt[i+1] += help->elem_cnt[i] + col_nnz[i] ;
  return help ;
}


#undef _COLBITPAT_CPP



