/*
   ----------------------------------------------------------------------------
   Author  : Uwe Geitner
   Hardware: IBM-compatible PC 386/40
   Software: g++-GNU-Compiler (v2.4) under Linux
   Date    : 14/07/95
   Content : - Definition of class ColBitpattern, derived from the base 
               class myBitpattern
   ----------------------------------------------------------------------------
*/

#ifndef _COLBITP_H
#define _COLBITP_H

#include "bitfeld.h"
#include "liste.h"
#include "MatLab.h"

class ColBitPattern : public myBitPattern
{
  private :
    int*    col_nnz ;      // array for the number of nonzeros (nnz/NNZ)
    int*    degree ;       // degree of the columns in the corresponding
                           // column incidence graph
    Liste*  nachb_liste ;  // contains for one vertex all neighbor vertices
                           // in the column incidence graph

  public :
    ColBitPattern() ;					
    ColBitPattern(unsigned int, unsigned int) ;					// Konstuktor
    ~ColBitPattern(void) ; 				
    void estimateDegree(void) ;
    MatLab_output* MatLab(void) const ;
    inline void incNNZinCol(int col)        { col_nnz[col]++ ; } ;
    inline void decNNZinCol(int col)        { col_nnz[col]-- ; } ;
    inline int  getNNZinCol(int col) const  { return col_nnz[col] ; } ;
    inline void setDegree(int col, int nr)  { degree[col] = nr  ; } ;
    inline int  getDegree(int col)   const  { return degree[col] ; } ;
    inline void incDegree(int col)          { degree[col]++ ; } ;
    inline void decDegree(int col)          { degree[col]-- ; } ;
    inline void addListElem(int col, int k) 
                { nachb_liste[col].addElement(k) ; } ;
    inline int  getListElem(int col, int k) 
                { return nachb_liste[col].getElement(k) ; } 
    inline int  sizeNaListe(int col) const  
                { return nachb_liste[col].getNumbElem() ; } ;
    friend ColBitPattern* allocColBP(unsigned int, unsigned int) ;
           // set the static variables (inherit from class myBitPattern) to 
           // the right size and allocate the dynamic memory
} ;

#endif


