/*
   ----------------------------------------------------------------------------
   Author  : Uwe Geitner
   Hardware: IBM-compatible PC 386/40
   Software: G++ GNU-Compiler under Linux
   Date    : 19/07/95
   Content : - Use the tape from ADOL-C and create with help from
               bitpattern a roworientated and colorientated
               representation of the jacobian matrix
             - returns a structure which contains the two necessary arrays
               for MatLab 
   Attention !!!!!
     The points after "ifdef conditional" are just in time not ready.     
   ----------------------------------------------------------------------------
*/

#define _CREATEPAT_CPP

#include "dvlparms.h" // Developers Parameters

// Necessary Includes

extern "C"  {
#include "usrparms.h"
#include "oplate.h"
#include "taputil2.h"
#include "taputil3.h"
}
#include "rowBitPat.h"
#include "colBitPat.h"
#include "MatLab.h"

/* Local Static Variables */

static short tag;

static   int ind_cnt ;
static   int dep_cnt ;
static   int for_location_cnt ;
static   int deaths ;

#ifdef inf_num
const double i_num=inf_num;
const double i_den=inf_den;
const double n_num=non_num;
const double n_den=non_den;
double InfVal;
double NoNum;
#endif

/****************************************************************************/
/* Create a BitPattern for the Gradient of y.                               */
/****************************************************************************/
MatLab_output* createPatternH(int tnum,                // Tapenumber
                             unsigned int depcheck,   // Number of dependent
                             unsigned int indcheck)   // Number of independent
{
  unsigned char operation;
  int tape_stats[11];

  int i, l, j ;
  locint arg , arg1, arg2, loc1, loc2, res ;
  locint size = 0 ; 
  int indexi  = 0 ;
  int indexd  = 0 ;

  int buffer;

  tag = tnum ;        // tag is global which specifies which tape to look at

  tapestats(tag,tape_stats);

  ind_cnt          = tape_stats[0];
  dep_cnt          = tape_stats[1];
  for_location_cnt = tape_stats[2];
  deaths           = tape_stats[3];
  buffer           = tape_stats[4];

  set_buf_size(buffer);

  if ((depcheck != dep_cnt)||(indcheck != ind_cnt))
    {
      printf("\n ADOL-C error: forward sweep on tape %d  aborted!\n",tag);
      printf("Number of dependent and/or independent variables passed to forward is\ninconsistant with number recorded on tape %d \n",tag);
      exit (-1);
    }

  RowBitPattern* Tbit ; // Bitpattern for independent, intermediate, dependent
  Tbit     = allocRowBP(for_location_cnt,ind_cnt) ;

  RowBitPattern* Hbit ; // Bitpattern for independent, intermediate, dependent
  Hbit     = allocRowBP(for_location_cnt,ind_cnt*(ind_cnt+1)) ;


  RowBitPattern* hessian_row ;           // Bitpattern for Hessian matrices
  hessian_row = allocRowBP(ind_cnt,ind_cnt) ;

  init_for_sweep(tag);                // Initialize the Forward Sweep

  operation=get_op_f();
  while (operation !=end_of_tape)
    {                                 // Switch Statement to perform operations
      switch (operation)
      {
         case end_of_op:
	   get_op_block_f();
           operation=get_op_f();      // Skip next operation, it's 
           break;                     // another end_of_op
         case end_of_int :
           get_loc_block_f();
           break;
         case end_of_val:
           get_val_block_f();
           break;
         case start_of_tape :
           break ; 
         case end_of_tape :
	   break;

         // Scalar Operations

         case assign_ind :  // Assign an adouble variable an independent
                            // float value.    ( <<= ) 
           res  =  get_locint_f() ;
           Tbit->setBit(res,indexi++) ;
           break ;

         case assign_dep :  // Assign a float variable a dependent 
                            // adouble value.  ( >>= )
           res  = get_locint_f() ;
           hessian_row->TRANS(Hbit,res) ;
	for (i=0; i<hessian_row->getPatSize(); i++)
	for (j=0; j<hessian_row->getPatSize(); j++)
               if (hessian_row->readBit(i,j))
               {
                 hessian_row->incNNZinRow(i) ;
                 hessian_row->incNNZinMat() ;
               }
           break ;

	 case assign_a :    // Assign an adouble variable an adouble value. ( = )
           arg = get_locint_f() ;
           res = get_locint_f() ;
           Tbit->EQUAL(res,arg) ;
           Hbit->EQUAL(res,arg) ;
           break ;
  
         case assign_d :    // Assign an adouble variable a float value. ( = )
           res = get_locint_f() ;
           break ; 
         
         case int_adb_a :   // Initialize an adouble with another adouble.
           arg = get_locint_f() ;
           res = get_locint_f() ; 
           Tbit->EQUAL(res,arg) ; 
           Hbit->EQUAL(res,arg) ; 
           break ;
 
         case int_adb_d :   // Initialize an adouble with a double value.
           res = get_locint_f() ;
           break ;

         case death_not :
           loc1 = get_locint_f() ;
           loc1 = get_locint_f() ;
           break ;

         case eq_plus_d :   // Add a floating point to an adouble. ( += )
           res  = get_locint_f() ;
           break ;

         case eq_plus_a :   // Add an adouble to another adouble. ( += ) 
           arg  = get_locint_f() ;
           res  = get_locint_f() ;
           Tbit->OR_EQ(res,arg) ;
           Hbit->OR_EQ(res,arg) ;
           break ;

         case eq_min_d :    // Subtract a floating point from an adouble. ( -= ) 
           res  = get_locint_f() ;
           break ;

         case eq_min_a :    // Subtract one adouble by another. ( -= )
           arg  = get_locint_f() ;
           res  = get_locint_f() ;
           Tbit->OR_EQ(res,arg) ;
           Hbit->OR_EQ(res,arg) ;
           break ;

         case eq_mult_d :   // Multiply an adouble by a float. ( *= )
           res  = get_locint_f() ;
           break ;

         case eq_mult_a :   // Multiply an adouble by another. ( *= )
           arg  = get_locint_f() ;
           res  = get_locint_f() ;
           Tbit->OR_EQ(res,arg) ;
           Hbit->ADD_EQ(res,arg,Tbit) ;
           break ;

         case plus_a_a :    // Add two adoubles. ( + )
           arg1 = get_locint_f() ;
           arg2 = get_locint_f() ;
           res  = get_locint_f() ;
           Tbit->OR(res,arg1,arg2) ;
           Hbit->OR(res,arg1,arg2) ;
           break ;

         case plus_d_a :    // Add an adouble and a double. ( + )
           arg  = get_locint_f() ;
           res  = get_locint_f() ;
           Tbit->EQUAL(res,arg) ;
           Hbit->EQUAL(res,arg) ;
           break ;

         case min_a_a :     // Subtraction of two adoubles. ( - )
           arg1 = get_locint_f() ;
           arg2 = get_locint_f() ;
           res  = get_locint_f() ;
           Tbit->OR(res,arg1,arg2) ;
           Hbit->OR(res,arg1,arg2) ;
           break ;

         case min_d_a :     // Subtract an adouble from a double ( - )
           arg  = get_locint_f() ;
           res  = get_locint_f() ;
           Tbit->EQUAL(res,arg) ;
           Hbit->EQUAL(res,arg) ;
           break ;

         case mult_a_a :    // Multiply two adoubles. ( * ) 
           arg1 = get_locint_f() ;
           arg2 = get_locint_f() ;
           res  = get_locint_f() ;
           Tbit->OR(res,arg1,arg2) ;
           Hbit->ADD(res,arg1,arg2,Tbit) ;
           break ;

         case mult_d_a :    // Multiply an adouble by a double. ( * ) 
           arg  = get_locint_f() ;
           res  = get_locint_f() ;
           Tbit->EQUAL(res,arg) ;
           Hbit->EQUAL(res,arg) ;
           break ;

         case div_a_a :     // Divide an adouble by an adouble. ( / )
           arg1 = get_locint_f() ;
           arg2 = get_locint_f() ;
           res  = get_locint_f() ;
           Tbit->OR(res,arg1,arg2) ;
           Hbit->ADD(res,arg1,arg2,Tbit) ;
           break ;

         case div_d_a :     // Division double - adouble. ( / )
           arg  = get_locint_f() ;
           res  = get_locint_f() ;
           Tbit->EQUAL(res,arg) ;
           Hbit->EQUAL(res,arg) ;
           break ;

         case sqrt_op :     // Compute sqrt of adouble.
           arg  = get_locint_f() ;
           res  = get_locint_f() ;
           Tbit->EQUAL(res,arg) ;
           Hbit->ADD(res,arg,Tbit) ;
           break ;

         case abs_val :     // Copmute fabs of adouble.
           arg  = get_locint_f() ;
           res  = get_locint_f() ;
           Tbit->EQUAL(res,arg) ;
           Hbit->EQUAL(res,arg) ;
           break ;

         case exp_op :      // exponent operation
           arg  = get_locint_f() ;
           res  = get_locint_f() ;
           Tbit->EQUAL(res,arg) ;
           Hbit->ADD(res,arg,Tbit) ;
           break ;

         case sin_op :      // sine operation
           arg1 = get_locint_f() ;
           arg2 = get_locint_f() ;
           res  = get_locint_f() ;
           Tbit->EQUAL(res,arg1) ;
           Hbit->ADD(res,arg,Tbit) ;
           break ;

         case cos_op :      // cosine operation
           arg1 = get_locint_f() ;
           arg2 = get_locint_f() ;
           res  = get_locint_f() ;
           Tbit->EQUAL(res,arg1) ;
           Hbit->ADD(res,arg,Tbit) ;
           break ;

         case asin_op :     // arcus-sine operation
           arg1 = get_locint_f() ;
           arg2 = get_locint_f() ;
           res  = get_locint_f() ;
           Tbit->EQUAL(res,arg1) ;
           Hbit->ADD(res,arg,Tbit) ;
           break ;

         case acos_op :     // arcus-cosine operation
           arg1 = get_locint_f() ;
           arg2 = get_locint_f() ;
           res  = get_locint_f() ;
           Tbit->EQUAL(res,arg1) ;
           Hbit->ADD(res,arg,Tbit) ;
           break ;

         case atan_op :     // arcus-tangens operation
           arg1 = get_locint_f() ;
           arg2 = get_locint_f() ;
           res  = get_locint_f() ;
           Tbit->EQUAL(res,arg1) ;
           Hbit->ADD(res,arg,Tbit) ;
           break ;                                    

         case log_op :      // logarithm operation  
           arg = get_locint_f() ;
           res = get_locint_f() ;
           Tbit->EQUAL(res,arg) ;
           Hbit->ADD(res,arg,Tbit) ;
           break ;

         case pow_op :      // power operation
           arg = get_locint_f() ;
           res = get_locint_f() ;
           Tbit->EQUAL(res,arg) ;
           Hbit->ADD(res,arg,Tbit) ;
           break ;

         // Vector Operations

         case int_av_av :   // Initialize an adouble vector with 
                            // another adouble vector.
           arg  = get_locint_f() ;
           size = get_locint_f() ;
           res  = get_locint_f() ; 
           for(l=0 ; l<size ; l++ )
             Tbit->EQUAL(res++,arg++) ;
             Hbit->EQUAL(res++,arg++) ;
           break ;

         case assign_dv :   // Assign an adouble variable vector
           arg  = get_locint_f() ;
           size = get_locint_f() ;
           break ;

         case assign_av :  
           arg  = get_locint_f() ;
           size = get_locint_f() ;
           res  = get_locint_f() ;
           for(l=0 ; l<size ; l++ )          
             Tbit->EQUAL(res++,arg++) ;
             Hbit->EQUAL(res++,arg++) ;
           break ;

         case assign_indvec : 
           size = get_locint_f() ;
           res  = get_locint_f() ;
           for(l=0 ; l<size ; l++)
             Tbit->setBit(res++,indexi++) ;                                 
           break ;

         case assign_depvec :                
           size = get_locint_f() ;     
           res  = get_locint_f() ; 
             hessian_row->TRANS(Hbit,res) ;  
             for (i=0; i<hessian_row->getPatSize(); i++)
             for (j=0; j<hessian_row->getPatSize(); j++)
               if (hessian_row->readBit(i,j))
               { 
                 hessian_row->incNNZinRow(i) ;
                 hessian_row->incNNZinMat() ;
               }
           break ;

         case eq_min_av :
         case eq_plus_av :
           arg  = get_locint_f() ;
           size = get_locint_f() ;
           res  = get_locint_f() ; 
           for(l=0 ; l<size ; l++){
             Tbit->OR_EQ(res,arg) ;
             Hbit->OR_EQ(res++,arg++) ;
           }
           break ;

         case eq_mult_av_a :
           arg  = get_locint_f() ;
           size = get_locint_f() ;
           res  = get_locint_f() ;
           for(l=0 ; l<size ; l++){
             Tbit->OR_EQ(res,arg) ;
             Hbit->ADD_EQ(res++,arg,Tbit) ;
           }
	 break ;

         case eq_mult_av_d :
           size = get_locint_f() ;
           res  = get_locint_f() ;
           break ;

         case plus_av_av :
         case sub_av_av  :
           arg1 = get_locint_f() ;
           arg2 = get_locint_f() ;
           size = get_locint_f() ;
           res  = get_locint_f() ;
           for(l=0 ; l<size ; l++){
             Tbit->OR(res,arg1,arg2) ;
             Hbit->OR(res++,arg1++,arg2++) ;
	   }
           break ;

         case dot_av_av :
           arg1 = get_locint_f() ;
           arg2 = get_locint_f() ;
           size = get_locint_f() ;
           res  = get_locint_f() ;
           for(l=0 ; l<size ; l++)
           {
             Tbit->OR_EQ(arg1,arg2) ;
             Hbit->ADD_EQ(arg1,arg2++,Tbit) ;
             Tbit->OR_EQ(res,arg1) ;
             Hbit->OR_EQ(res,arg1++) ;
           }
           break ; 

         case mult_d_av :
           arg  = get_locint_f() ;
           size = get_locint_f() ;
	   res  = get_locint_f() ;
	   for(l=0 ; l<size ; l++){
             Tbit->OR_EQ(res,arg) ;
             Hbit->OR_EQ(res++,arg++) ;
           }
           break ;

         case mult_av_a :
         case div_av_a  :
           size = get_locint_f() ;
           arg1 = get_locint_f() ;
           arg2 = get_locint_f() ;
           res  = get_locint_f() ;
           for(l=0 ; l<size ; l++){
             Tbit->OR(res,arg1,arg2) ;
             Hbit->ADD(res++,arg1++,arg2,Tbit) ;
	   }
           break ;  

         case mult_a_av :
           size = get_locint_f() ;
           arg2 = get_locint_f() ;
           arg1 = get_locint_f() ;
           res  = get_locint_f() ;
           for(l=0 ; l<size ; l++){
             Tbit->OR(res,arg1,arg2) ;
             Hbit->ADD(res++,arg1,arg2++,Tbit) ;
           }
           break ;
// added by Arun

        case eq_d_a :
           arg2 = get_locint_f() ;
        break ;
        case eq_a_a :
           arg2 = get_locint_f() ;
           arg1 = get_locint_f() ;
        break ;
        case neq_d_a :
           arg1 = get_locint_f() ;
        break ;
        case neq_a_a :
           arg2 = get_locint_f() ;
           arg1 = get_locint_f() ;
        break ;
        case gt_a_a :
           arg2 = get_locint_f() ;
           arg1 = get_locint_f() ;
        break ;
        case gt_d_a :
           arg1 = get_locint_f() ;
        break ;
        case lt_a_a :
           arg2 = get_locint_f() ;
           arg1 = get_locint_f() ;
        break ;
        case lt_d_a :
           arg1 = get_locint_f() ;
        break ;
        case ge_d_a :
           arg2 = get_locint_f() ;
        break ;
        case ge_a_a :
           arg2 = get_locint_f() ;
           arg1 = get_locint_f() ;
        break ;
        case le_d_a :
           arg2 = get_locint_f() ;
        break ;
        case le_a_a :
           arg2 = get_locint_f() ;
           arg1 = get_locint_f() ;
        break ;

#ifdef conditional
         case cond_assign :
         case cond_assign_s :
         case subscript :
         case subscript_l :
         case subscript_ld :
         case m_subscript :
         case m_subscript_l :
         case m_subscript_ld :
         break ;
#endif    

         default :
           cout << "ADOL-C error: Fatal error in createPat() for " ;
           cout << operation << "\n" ;
           break ;
      }              

      operation = get_op_f() ;
    } 
  end_sweep() ; 

  delete Tbit ;
  delete Hbit ;
  hessian_row ;
  ColBitPattern*   hessian_col = hessian_row->row2col() ;
  MatLab_output*  matlab_out ;
  matlab_out = hessian_col->MatLab() ;

  delete hessian_row ;
  delete hessian_col ;


  return matlab_out ;
}

#undef _CREATEPAT_CPP


