/*
   ----------------------------------------------------------------------------
   Author  : Uwe Geitner
   Hardware: IBM-compatible PC 386/40
   Software: g++-GNU-Compiler (v2.4) under Linux
   Date    : 18/07/95
   Content : - Menue for the 3 Methods
             - Definition ot the Function F: R(n) --> R(n) ;
               Simplified form of the neutron kinetics problem by Carver
               and MacEwen ; see e.g. the paper "Software for estimating
               sparse jacobian matrices" by Coleman/Mor
             - Call to the corresponding Function 
   ----------------------------------------------------------------------------
*/

#include "adouble.h"
#include "adutils.h"
#include "createPat.h"
#include "MatLab.h"
#include <stream.h>

main() 
{
  int  n ;                  // Size for jacobian matrix (nxn)                    
  int  l ;                  // Size for the 9 undermatrices n=3*l
  int  num_of_choice = 0 ;

  cout << "\n    Which Approach ? Give the corresponding number !\n\n" ;
  cout << "          1)    Coleman/More\n" ;
  cout << "          2)    Newsam/Ramsdell Classic\n" ;
  cout << "          3)    Newsam/Ramsdell Modern\n" ;
  while (!((num_of_choice == 1)||(num_of_choice == 2)||(num_of_choice == 3)))
  {
    cout << "\n    Number  : " ;
    cin  >> num_of_choice ;
  }

  cout << "\n    Size of l = " ;
  cin  >> l ;
  n = 3*l ;

   double*  py = new  double[n] ;   // decleare dep. passive variable pointer
  adouble*   y = new adouble[n] ;   // decleare dep. active  variable pointer 
   double*  px = new  double[n] ;   // decleare ind. passive variable pointer
  adouble*   x = new adouble[n] ;   // decleare ind. active  variable pointer

  trace_on(1) ;                     // start tracing
  
  for(int i=0; i<n; i++)
  {
    px[i] = i ;                     // values for passive variables
     x[i] <<= px[i] ;               // select independent variables
  }

  y[0]     = x[0]   + x[1]     + x[l] ;            
  y[l-1]   = x[l-2] + x[l-1]   + x[2*l-1] ;  
  y[l]     = x[0]   + x[l]     + x[l+1] + x[2*l] ; 
  y[2*l-1] = x[l-1] + x[2*l-2] + x[2*l-1] + x[3*l-1] ;
  y[2*l]   = x[0]   + x[l]     + x[2*l] ; 
  y[3*l-1] = x[l-1] + x[2*l-1] + x[3*l-2] + x[3*l-1] ;

  for(i=1; i<l-1; i++)
  {
    y[i]     = x[i-1] + x[i]     + x[i+1]     + x[l+i] ;
    y[l+i]   = x[i]   + x[l-1+i] + x[l+i]     + x[l+1+i] + x[2*l+i] ;
    y[2*l+i] = x[i]   + x[l+i]   + x[2*l-1+i] + x[2*l+i] ;
  }   

  for (i=0; i<n; i++)
    y[i] >>= py[i] ;                 // select dependent variables

  trace_off(1) ;                     // complete tape on adoltape
  
  delete[] px ;
  delete[] py ;
  delete[]  x ;
  delete[]  y ;

  MatLab_output*   two_arrays4you ;
  two_arrays4you = createPattern(1,n,n) ;

  cout << "Values of the first array \n\n" ;
  for (i=0; i<two_arrays4you->size_array_elem+1; i++)
    cout << "elem[" << i << "] :\t" << two_arrays4you->elem_cnt[i] << "\n" ;

  cout << "Values of the second array \n\n" ;
  for (i=0; i<two_arrays4you->size_array_row; i++)
    cout << "inedx_row[" << i << "] :\t" << two_arrays4you->row_indices[i] << "\n" ;

  destrMatLab(two_arrays4you) ;
}





