/*
   ----------------------------------------------------------------------------
   Author  : Uwe Geitner
   Hardware: IBM-compatible PC 386/40
   Software: g++-GNU-Compiler (v2.4) under Linux
   Date    : 13/07/95
   Content : - Implementation of the methods and the friend functions for the 
               class RowBitPattern (base class is class myBitPattern) :
               RowBitPattern::RowBitPattern(),
               RowBitPattern::RowBitPattern(unsigend int, unsigned int),
               RowBitPattern::~RowBitpattern(),
               ColBitPattern* RowBitPattern::row2col(void),
               friend RowBitPattern* allocRowBP(unsigned int, unsigned int),
   ----------------------------------------------------------------------------
*/

#define _ROWBITPAT_CPP

#include "rowBitPat.h"
#include <memory.h>
#include <stdio.h>

RowBitPattern::RowBitPattern()
{
  arr_size = 0 ;
  pat_size = 0 ;
  blocks   = 0 ;
  NNZinMat = 0 ;
  bits     = NULL ;
  row_nnz  = NULL ;
}

RowBitPattern::RowBitPattern(unsigned int rows, unsigned int psize)
{
  arr_size = rows ;
  pat_size = psize ;
  blocks   = (pat_size + 8*sizeof(int)-1) / (8*sizeof(int)) ;
  NNZinMat = 0 ;
  row_nnz  = new int[arr_size] ;
  memset(row_nnz, 0, arr_size * sizeof(int)) ;
  bits     = new int*[arr_size] ;
  for (int i=0; i<arr_size; i++)
  {
    bits[i] = new int[blocks] ;
    memset(bits[i], 0, blocks * sizeof(int)) ;
  } 
}

RowBitPattern* allocRowBP(unsigned int rows, unsigned int psize)
{
  RowBitPattern*  help = new RowBitPattern ;
  help->arr_size = rows ;
  help->pat_size = psize ;
  help->blocks   = (help->pat_size + 8*sizeof(int)-1) / (8*sizeof(int)) ;
  help->NNZinMat = 0;
  help->row_nnz  = new int[help->arr_size] ;
  memset(help->row_nnz, 0, help->arr_size * sizeof(int)) ;
  help->bits = new int*[help->arr_size] ;
  for (int i=0; i<help->arr_size; i++)
  {
    help->bits[i] = new int[help->blocks] ;
    memset(help->bits[i], 0, help->blocks * sizeof(int)) ;
  }
  return help ;
}

RowBitPattern::~RowBitPattern()
{
  if (row_nnz != NULL)  delete[] row_nnz ;
  if (bits != NULL)
  {
    for (int i=0; i<arr_size; i++)
      if (bits[i] != NULL)  delete[] bits[i] ;
    delete[] bits ;
  }
}

ColBitPattern* RowBitPattern::row2col(void)
{
  ColBitPattern* help = allocColBP(pat_size, arr_size) ;
  help->setNNZinMat(NNZinMat) ;
  for (int i=0; i<arr_size; i++)
    for (int j=0; j<pat_size; j++)
      if (readBit(i,j))
      {
	help->setBit(j,i) ;
      	help->incNNZinCol(j) ;
      }
  return help;
}

#undef _ROWBITPAT_CPP

