function [x,it,gc,val]=main(func,tol,itbound,x,u,l,fdata)
%  
%  A trust region method for minimizing a nonlinear function subject to bounds
%
%  tol - tolerance for solving problem
%  itbound - maximum number of iterations
%  March 28, 98, Yuying Li
%
mu = 0.25;
n = length(x);

val = inf; fc =0; gc=0;
[x]=Perturb(x,l,u);

[newval,g,H]=feval(func,x,fdata);
c=g-H*x;
gc=gc+1; fc=fc+1;

[v,dv]=DefineV(g,x,l,u);
sqrtv = sqrt(abs(v));
hatg = sqrtv.*g; posdef =0;

delta0 = (sqrt(sum(min(x-l,u-x)))+1);
deltaU = delta0; deltaL=1;
delta0 = min(0.1*norm(g),delta0);
delta = delta0;
success =0;qval=-1;
%
%  Main loop
%
for it = 1:itbound
    vdelta(it)=delta;
    diff = abs(val-newval);
    newval = newval
    vval(it) = val; VGnorm = norm(v.*g,inf);
vgnorm_qval=[VGnorm,qval]
    vvg(it) = VGnorm;
    if (qval >-tol | VGnorm <tol )
         break;
    end
    val = newval;
%
%   form the quadratic model hat{psi}_k(w)
%
    D =sparse(1:n,1:n,sqrtv);
    DS =sparse(1:n,1:n,ones(n,1));
    B = D*H*D + sparse(1:n,1:n,(g.*dv));
    B=(B+B')/2; 
%
%   Solve a full space trust region problem
%
    [ss,posdef,rhs,v1c] = FullTrust(B,sqrtv,hatg,delta);
    vv1c(it) = v1c;
    vpos(it) = posdef;
    s = sqrtv.*ss;
%
%   determine the direction
%
    [s,ss,qval,gTs,rho_c]=SgTrust ...
                    (rhs,hatg,sqrtv,s,ss,x,l,u,g,B,delta,VGnorm,H);
    vrho_c(it) = rho_c;

    [newval]=feval(func,x+s,fdata);
    fc = fc+1;
    sTCs = ss'*(g.*dv.*ss);
    rho_f=(newval-val+0.5*sTCs)/qval;
    vrho_f(it) = rho_f;
%
%   Determine the trust region size for the next iteration
%
    [delta]=UpdateDelta(rho_f,rho_c,delta,ss,deltaL,deltaU);
    success =0;
    if( rho_f > mu  )
        success =1;
        x=x+s;
        [x]=Perturb_eps(x,l,u);
        [val,g,H]=feval(func,x,fdata);
        gc=gc+1;
        [v,dv]=DefineV(g,x,l,u);
        sqrtv = sqrt(abs(v));
        hatg = sqrtv.*g;
    else
        disp('fail');
        newval = val;
    end;
end

val = vval(it);
Number_of_Iterations =it
Degeneracy = min(abs(v)+abs(g));
if (Degeneracy < 10^(-5)) Degeneracy = Degeneracy,end;
posdef = posdef
VGnorm = VGnorm

return
