   function[v,posdef] = newtorneg1(HMTX,DM,DG,g)
%
%  THIS ROUTINE DETERMINES THE NEWTON STEP, Mv = -g,
%  WHERE  M = DM*HMTX*DM + DG;
%  IF M IS FOUND TO BE POSITIVE DEFINITE. OTHERWISE,
%  A DIRECTION OF NON-POSITIVE CURVATURE IS RETURNED IN v.
%  posdef = 1 INDICATES M IS POSITIVE DEFINITE (AND
%  v1 IS THE NEWTON STEP, posdef = 0 INDICATES M IS
%  NOT POSITIVE DEFINITE AND v1 IS A DIRECTION ON POSITIVE
%  CURVATURE.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%      Tom Coleman, Cornell University.  December 5, 1991.    %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
   n = length(g); v = zeros(n,1); 
   M = DM*HMTX*DM + DG;
%
% A SPARSE CHOLESKY FACTORIZATION OF M IS ATTEMPTED
% USING THE SYMMETRIC MINIMUM DEGREE ORDERING. ANOTHER
% FILL-REDUCING ORDERING IS POSSIBLE BY SIMPLY REPLACING
% THE NEXT LINE, 'p=symmmd(M)' USING AN ALTERNATIVE ORDERING
% AVAILABLE IN MATLAB.
   p = symmmd(M);
   NEWT = 1;
   [R,info] = chol(M(p,p));
   if info == 0
     temp = R'\g(p); v(p,1) = -R\temp;
   else
     if info == 1
       v(1) = 1;
     else
       j = info -1;
       v(1:j,1) = -R(1:j,1:j)\R(1:j,j+1); v(j+1,1) = 1;
     end
     v(p) = v;
   end
   vg = v'*g;
   sgn = sign(vg) + (vg == 0);
   v = -sgn*v;
   if info == 0, posdef = 1; else, posdef = 0; end
%
% LAST LINE OF newtorneg
