  function R = precon(type,DM,DG);
  global HMTX
%
% PRODUCE SPARSE NONSINGULAR UPPER TRIANGULAR MATRIX R SUCH THAT 
% MATRIX R'R IS A PRECONDITIONER FOR THE MATRIX
%
%       M = DM*H*DM + DG
%
% WHERE DM AND DG ARE NON-NEGATIVE SPARSE DIAGONAL MATRICES,
% H IS A SYMMETRIC MATRIX. H MAY BE AVAILABLE
% THROUGH global VARIABLES, E.G., H = HMTX, OR H
% MAY ONLY BE IMPLICITLY AVAILABLE THROUGH FUNCTION
% Hmult, A MATRIX-VECTOR MULTIPLIER. PARAMETER type
% IS USED TO CHOOSE THE TYPE OF PRECONDITIONER DESIRED --
% CARE MUST BE TAKEN TO ENSURE COMPATIBILITY BETWEEN 
% THE PRECONDITIONER type CHOSEN AND THE AVAILABILITY
% OF H.
%
% MATRIX M =  DM*H*DM + DG IS NOT ASSUMED TO BE POSITIVE DEFINITE
%
% THE USER CAN ADD ADDITIONAL PRECONDITIONER types.

  
  n = length(DM);
%
% IF type IS POSITIVE THEN MATRIX H IS EXPLICITLY ACCESSED
% THROUGH GLOBAL VARIABLE HMTX TO CONSTRUCT A PRECONDITIONER.
  if type > 0
%
%    IF type = 1 THEN A DIAGONAL PRECONDITIONER IS CONSTRUCTED. 
     if type == 1
        d = full(diag(HMTX)); D = sparse(1:n,1:n,d);
        D = DM*D*DM+DG; d = abs(diag(D));
        d = full(sqrt(d));
        d(d==0) = ones(length(d(d==0)),1);
        R = sparse(1:n,1:n,d);
     end
%
%    IF type == 2 THEN A TRIDIAGONAL PRECONDITIONER IS CONSTRUCTED.
     if type == 2
        A = HMTX - triu(HMTX,2) - tril(HMTX,-2);
        A = DM*A*DM +DG;
        [R,info]  = chol(A);
        if info > 0
           d = full(sqrt(abs(diag(A))));
           d(d==0) = ones(length(d(d==0)),1);
           R = sparse(1:n,1:n,d);
        end
     end
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%    MODIFY THE FOLLOWING LINE IF MORE POSITIVE types ARE ADDED
     if type > 2, R = speye(n); end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
  else
%
%    MATRIX H IS NOT EXPLICITLY ACCESSED TO CONSTRUCT
%    A PRECONDITIONER --  GLOBAL PARAMETER HMTX IS NOT NEEDED.
%
%    IF type = -1 A DIAGONAL PRECONDITIONER IS CONSTRUCTED 
%    WITHOUT ACCESSING HMTX
     if type == -1
        dir = rand(n,5); diff = Hmult(dir);
        d = zeros(n,1);
        for i=1:n
            d(i,1) = abs((dir(i,:)*diff(i,:)')/(dir(i,:)*dir(i,:)'));
        end
        D = sparse(1:n,1:n,d);
        D = DM*D*DM+DG; d = full(sqrt(abs(diag(D))));
        d(d==0) = ones(length(d(d==0)),1);
        R = sparse(1:n,1:n,d);
     end
%
%    IF type = -2 A TRIDIAGONAL PRECONDITIONER IS CONSTRUCTED
%    WITHOUT ACCESSING HMTX
     if type == -2
        A = spalloc(n,n,3*n);
        dir= rand(n,3); diff = Hmult(dir);
        vec = zeros(1:3,1);
        for i=2:n-1
            MAT = dir(i-1:i+1,:);
            rhs = diff(i,:)';
            vec = MAT'\rhs;
            A(i,i-1:i+1) = vec';
        end
        MAT = dir(1:2,1:2); rhs = diff(1,1:2)';
        vec = MAT'\rhs;
        A(1,1:2) = vec';
        MAT = dir(n-1:n,2:3); rhs = diff(n,2:3)';
        vec = MAT'\rhs;
        A(n,n-1:n) = vec';
        A = .5* (A+A'); 
        [R,info]  = chol(A);
        if info > 0
           d = full(sqrt(abs(diag(A))));
           d(d==0) = ones(length(d(d==0)),1);
           R = sparse(1:n,1:n,d);
        end
     end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%    MODIFY THE FOLLOWING LINE IF MORE NEGATIVE types ARE ADDED
     if type < -2, R = speye(n); end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  end
%
% USER CAN ADD MORE PRECONDITIONER OPTIONS 
% WITH THE USE OF `elseif'. ALTERNATIVELY
% THE USER CAN REPLACE FUNCTION precon WITH
% THE USERS CUSTOMIZED PRECONDITIONER ROUTINE precon.
% BE SURE THE PRECONDITIONER
% DEFINITION IS COMPATIBLE WITH THE AVAILABILITY OF H.
%
  end
%
% LAST LINE OF precon
