function [iter,fc,val,Tflop]=QuadTrust(func,grad,hess,tol,itbound,x,u,l,tsub)
%  
%  A trust region method for minimizing a quadratic subject to bounds
%
%  tol - tolerance for solving problem
%  itbound - maximum number of iterations
%  March 10, 93, Yuying Li
%
mu = 0.25;
n = length(x);
tsub = min(tsub,n);

val = inf; fc =0; gc=0;
[x]=Perturb(x,l,u);

newval = eval( [func,'(x)' ] );
g = eval([grad,'(x)']); H = eval([hess,'(x)']);
c=g-H*x;
gc=gc+1; fc=fc+1; p=symmmd(H);

[v,dv]=DefineV(g,x,l,u);
sqrtv = sqrt(abs(v));
hatg = sqrtv.*g; posdef =0;

%delta0 = min(0.1*norm(g),2*sqrt(n))
delta0 = 0.5*(sqrt(sum(min(x-l,u-x)))+1)
delta = delta0; deltaU = delta0; deltaL=1;

%
%  Main loop
%
flop1 = flops;
for it = 1:itbound
    vdelta(it)=delta;
    val = newval;
    vval(it) = val; VGnorm = norm(v.*g,inf)
    vvg(it) = VGnorm;
    if ( VGnorm < tol & posdef ) 
         break;
    end
%
%   form the quadratic model hat{psi}_k(w)
%
    D =sparse(1:n,1:n,sqrtv);
    DS =sparse(1:n,1:n,ones(n,1));
    B = D*H*D + sparse(1:n,1:n,(g.*dv));
    B=(B+B')/2; 
%
%   Solve a subspace trust region problem
%
%   delta = max(sqrt( sum(min(x-l,u-x))),1);
    [ss,posdef,rhs,v1c] = SolveTrust(B,sqrtv,p,hatg,delta,tsub);
    vv1c(it) = v1c;
%   [ss,posdef,rhs] = Solve2(B,sqrtv,p,hatg,delta,tsub);
    s = sqrtv.*ss;
%
%   determine the direction
%
    [s,ss,qval,gTs,rho_c,gamma]=Sg_or_Trust(rhs,hatg,sqrtv,s,ss,x,l,u,g,B,delta,VGnorm);
    vrho_c(it) = rho_c;

    newval = eval( [func,'(x+s)' ] );
    sTCs = ss'*(g.*dv.*ss);
    rho_f=(newval-val+0.5*sTCs)/qval;
    rho_f=1;
%
%   Determine the trust region size for the next iteration
%
    [delta]=UpdateDelta(rho_f,rho_c,delta,ss,gamma,deltaL,deltaU)
    if( rho_f > mu )
        x=x+s;
        [x]=Perturb_eps(x,l,u);
        g = eval([grad,'(x)']); H = eval([hess,'(x)']);
        [v,dv]=DefineV(g,x,l,u);
        sqrtv = sqrt(abs(v));
        hatg = sqrtv.*g;
    else
        disp('fail');
        newval = val;
        keyboard
    end;
end

Tflop = flops-flop1
val = vval(it);
Optimal_val = val
Number_of_Iterations =it
keyboard
Degeneracy = min(abs(v)+abs(g))
save /usr/fsys/amma/a/yuying/matlab/nonlinear/bound/probs/Ucsln x
clear B
return
