function [iter,fc,val,Tflop]=Reflect(func,grad,hess,tol,itbound,x,u,l,tsub)
%  
%  An interior point method to solve
%        min  f(x)
%           l<= x <=u
%  A trust region problem is solved to determine
%  the search direction followed by a line search.
%
%	Feb, 93
%
%  tol - tolerance for solving problem
%  itbound - maximum number of iterations
%
opt=1; posdef =0;
dellow = 1.; delup = 10^3;
deltaL=10^(-6);  mu = 0.25;
n = length(x);
tsub = min(tsub,n);

val = inf; fc =0; gc=0;
[x]=Perturb(x,l,u);

newval = eval( [func,'(x)' ] );
g = eval([grad,'(x)']); H = eval([hess,'(x)']);
c=g-H*x;
gc=gc+1; fc=fc+1; p=symmmd(H);

[v,dv]=DefineV(g,x,l,u);
sqrtv = sqrt(abs(v));
hatg = sqrtv.*g;
delta = min(0.1*norm(g),2*sqrt(n));
delta = max(sqrt( sum(min(x-l,u-x))),1)
%
%  Main loop
%
flop1 = flops;
for it = 1:itbound
    vdelta(it)=delta;
    val = newval;
    vval(it) = val; VGnorm = norm(v.*g,inf)
    vvg(it) = VGnorm;
    if ( VGnorm < tol & posdef ) 
         break;
    end
%
%   form the quadratic model hat{psi}_k(w)
%
    D =sparse(1:n,1:n,sqrtv);
    DS =sparse(1:n,1:n,ones(n,1));
    B = D*H*D + sparse(1:n,1:n,(g.*dv));
    B=(B+B')/2; 
%
%   Solve a subspace trust region problem
%
%   delta = max(dellow,norm(v)); delta = min(delta,delup);
%   [sx,posdef,Zsize,v1] = dirn2(B,D,p,hatg,delta,tsub,opt);
%   ssx = sx./sqrtv;
%delta = max(sqrt(sum(min(x-l,u-x))),1)
delta =norm(v); delta = min(delta,10^3); delta = max(delta,1);
    [ssx,posdef,rhs,v1c] = SolveTrust(B,sqrtv,p,hatg,delta,tsub);
    vv1c(it) = v1c;
%   [ssx,posdef] = Solve2(B,sqrtv,p,hatg,delta,tsub);
    sx = sqrtv.*ssx;
    strg = g'*sx;
    [xnew] = approx_line(sx,H,c,strg,VGnorm,ones(n,1),x,u,l,n,posdef);
%
%   determine the direction
%
    sx=xnew-x;
    vnormDs(it) = norm(sx./sqrtv);
%   [sx,ssx,qval,gTs]=Sg_or_Trust(hatg,sqrtv,sx,ssx,x,l,u,g,B,delta,VGnorm);

    newval = eval( [func,'(x+sx)' ] )

%   rho=(newval-val+0.5*sTCs)/qval
    rho=1;
%
%   Determine the trust region size for the next iteration
%
%   [delta]=UpdateDelta(rho,delta,gTs,sx,qval,sTCs,val);
    if( rho > mu )
        x=x+sx;
        [x]=Perturb_eps(x,l,u);
        g = eval([grad,'(x)']); H = eval([hess,'(x)']);
        [v,dv]=DefineV(g,x,l,u);
        sqrtv = sqrt(abs(v));
        hatg = sqrtv.*g;
    end;
end

Tflop = flops-flop1
Optimal_val = vval(it)
Number_of_Iterations =it
iter = it;
Degeneracy = min(abs(v)+abs(g))
keyboard
save /usr/fsys/amma/a/yuying/matlab/nonlinear/bound/probs/Ucsln x
clear B
return
