function [sx,posdef,rhs] = Solve2(M,sqrtv,p,sgr,delta,tsize);
%
%  find sx for the scaled trust region problem satisfying convergence criteria
%
n = length(sqrtv);
[v1,posdef] = newtorneg(M,p,sgr);
%
% Newton step exists and is within the trust region 
%
if posdef == 1 & norm(v1) <= delta
   disp('Newton Step')
   sx = v1; 
   rhs = sgr;
   return;
end;

if posdef ==1 
%
%  form a two dimensional subspace if possible
%
   disp('Trust Region : Positive Definite')
   Z = zeros(n,2);
   Z = [v1,sgr];
   v1 = v1/norm(v1); v2 = sgr/norm(sgr);
   Z(:,1) = v1; v2 = v2 - v1*(v1'*v2);
   nrmv2 = norm(v2);
   if nrmv2 > 0
      v2 = v2/nrmv2; Z(:,2) = v2; Zsize = 2;
   else
      Zsize = 1;
   end
else
%
%  indefinite: form a two dimensional subspace if possible
%
   v2 = sqrtv.*sign(sgr);
   v2 = v2/norm(v2);
   v2 = v2 - v1*(v1'*v2);
   v2 = v2/norm(v2);
   Z(:,1) = v1; Z(:,2) = v2;
   Zsize = 2;
end
%
%
% solve the two-or-three dimensional trust region problem 
%
MM = full(Z'*M*Z);
rhs=full(Z'*sgr);
[sx2,qpval,po,key,fcnt,lambda] = trust_tom(rhs,MM,delta);
sx= Z*sx2;
rhs = Z*rhs;
return
