function [sx,posdef,rhs,v1c] = SolveTrust(M,sqrtv,p,sgr,delta,tsize,os);
%
%  find sx for the scaled trust region problem satisfying convergence criteria
%
n = length(sqrtv);
[v1,posdef] = newtorneg(M,p,sgr);
v1c =norm(v1./sqrtv,inf);
%
% Newton step exists and is within the trust region 
%
if posdef == 1 & norm(v1) <= delta
   disp('Newton Step')
   sx = v1; 
   rhs = sgr;
   return;
end;

if posdef ==1 
%
%  form a two dimensional subspace if possible
%
%  disp('Trust Region : Positive Definite')
   Z = zeros(n,2);
   Z = [v1,sgr];
   v1 = v1/norm(v1); v2 = sgr/norm(sgr);
   Z(:,1) = v1; v2 = v2 - v1*(v1'*v2);
   nrmv2 = norm(v2);
   if nrmv2 > 0
      v2 = v2/nrmv2; Z(:,2) = v2; Zsize = 2;
   else
      Zsize = 1;
   end
else
%
%  indefinite: form a two dimensional subspace if possible
%
    reorth = 0;
    sgrs = sgr;
    if ( tsize > 1 )
       [Y,alpha,beta] = lanczos(M,sgrs,tsize,reorth);
       tsize=length(alpha);
       [meig,normT]=bisect(alpha,beta);
       [w]=inverse(normT,tsize,meig,alpha,beta);
       w = Y(:,1:tsize)*w; w=w/norm(w);
       weig=full(w'*M*w);
    else
       w = zeros(n,1); weig = 0;
    end
    Z=zeros(n,2);
    normsgrs =norm(sgrs);
    Z(:,1) = sgrs/normsgrs;
    ww = Z'*w; w = w - Z*ww;
    sec_col=0;
    if norm(w) > eps
       sec_col=1;
       w = w/norm(w);
       Z(:,2) = w;
       end
    end
%   add the negative curvature direction from partial Chol if possible
    normv1 = norm(v1);
    if ( weig >= 0 )
         v1=v1/normv1;
         ww = Z'*v1; v1 = v1 - Z*ww;
         if norm(v1) > eps
            v1 = v1/norm(v1);
            if( sec_col == 1 )
                Z(:,3) = v1;
            else
                Z(:,2) = v1;
            end;
        end
    end
%
    while ( weig >=0 & normv1<eps & tsize < n )
          disp('more Lanczos iteratios  necessary');
          tsize=min(tsize*2,n);
          [Y,alpha,beta] = lanczos(M,sgrs,tsize,reorth);
          tsize=length(alpha);
          [meig,normT]=bisect(alpha,beta);
          [w]=inverse(normT,tsize,meig,alpha,beta);
          w = Y*w; w=w/norm(w);
          weig=full(w'*M*w);
          Z=zeros(n,2);
          Z(:,1) = sgrs/normsgrs;
          ww = Z'*w; w = w - Z*ww;
          if norm(w) > eps
             w = w/norm(w);
             Z(:,2) = w;
          end
    end
end
%
% solve the two-or-three dimensional trust region problem 
%
MM = full(Z'*M*Z);
Zsize = size(MM)
rhs=full(Z'*sgr);
%[sx2,qpval,po,key,fcnt,lambda] = trust_tom(rhs,MM,delta);
[sx2,qpval,po,key,fcnt,lambda] = trust(rhs,MM,delta);
if ( norm(sx2) > 1.2*delta) nrms = norm(sx2), keyboard,end;
sx= Z*sx2;
%rhs = Z*rhs;
return
