function[alpha,x]=line2(s,H,c,strg,theta,osigma,ox,u,l,nbr,posdef)
%
%  THIS ROUTINE  PERFORMS AN APPROXIMATE MINIMIZATION OF A
%  PIECEWISE QUADRATIC FUNCTION AS DESCRIBED IN COLEMAN & LIU ??.
%  INTENDED FOR USE IN SOLVING THE PROBLEM: min {c'x + 1/2 x'Hx: l <= x  <= u}.
%  THE APPROACH DESCRIBED IN ?? TRANSFORMS THIS CONSTRAINED PROBLEM INTO
%  AN UNCONSTRAINED MINIMIZATION OF A PIECEWISE QUADRATIC FUNCTION OF A NEW
%  (UNCONSTRAINED) VARIABLE y. THE PURPOSE OF THIS ROUTINE IS TO DETERMINE 
%  AN ACCEPTABLE STEP LENGTH alpha (THE CALLING PROGRAM UPDATES
%  y ALONG DIRECTION s). 
%  CONVENIENTLY, approx_line ALSO UPDATES THE VECTOR x.
%
%  THIS SUBROUTINE REQUIRES O(nnz(H) + k*n) WORK.
%  
%
%  INPUT VARIABLES (UNCHANGED):
%   s: THE SEARCH DIRECTION. ON RETURN THE VARIABLE y (CALLING PROGRAM)
%      WILL BE UPDATED IN THE CALLING PROGRAM, y = y + alpha * s.
%   strg: THE INITIAL INNER-PRODUCT BETWEEN THE GRADIENT AND s.
%   theta: BOUNDED BEWTEEN 0 and 1. INDICATES 'NEARNESS TO OPTIMALITY' , I.E.,
%          theta GOES TO ZERO AS x GOES TO A SOLUTION.
%   osigma: SIGN VECTOR.
%   nbr: UPPER BOUND ON THE NUMBER OF BREAKPOINTS ALLOWED TO BE CROSSED.
%   u: UPPER BOUNDS. COMPONENTS EQUAL EITHER 1 OR inf.
%   l: LOWER BOUNDS. COMPONENTS EQUAL EITHER 0 OR -inf.
%   
%  OUTPUT VARIABLES:
%   x: THE UPDATED VALUE OF THE FEASIBLE VECTOR x. l <= x <= u.
%   sigma: A SIGN VECTOR (UPDATED) RELATING x TO y (CALLING PROGRAM).
%   alpha: THE STEP LENGTH USED TO UPDATE VARIABLE y (CALLING PROGRAM).
%   k: THE ACTUAL NUMBER OF BREAKPOINTS CROSSED. k <= nbr.
%   done: A FLAG INDICATING THE EXIT CRITERION SATISFIED.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%      Tom Coleman, Cornell University.  December 4, 1991.    %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%      INITIALIZATIONS
       taubar = .85;
       done = 0; k = 0; num = strg; ssum = 0;
       oldalf=0; step = 0; 
       x = ox;  sigma = osigma;
       w = s.*(H*(s.*sigma));
       denom = sigma'*w;
%
%      COMPUTE BREAKPOINTS BR
       BR = max(full(l-x)./(full(sigma).*full(s)), full(u-x)./(full(sigma).*full(s)));
       BR(BR<0) = inf*ones(length(BR(BR<0)),1);
%
%      MAIN LOOP. PROCEED THROUGH THE BREAKPOINTS IN A LEFT-TO-RIGHT FASHION.
       while done == 0,
          k=k+1;
          [alpha,beta] = min(BR);
          if posdef == 1
             tau=max(taubar,1-theta);
           else
             tau = taubar;
          end
          step=alpha-oldalf;
             if denom > 0
               opt=-num/denom;
             else
               opt = inf;
             end
             if opt < step    % A MINIMIZER IS WITHIN THE CURRENT INTERVAL %  
                alpha=oldalf+opt;
                x = x + opt*(sigma.*s);
                k = k
                done = 1;
             else            % CHECK THE NEXT BREAKPOINT %
                osig = sigma;
                sig = sigma(beta);
                sigma(beta) = -sigma(beta);
                BR(beta) = BR(beta) + abs(u(beta)/s(beta));
                num = num - 2*sig*s(beta)*(H(beta,:)*x);
                num = num - 2*sig*s(beta)*c(beta) + step*sigma'*w;
                w = w - s.*(H(:,beta)*(2*s(beta)*sig));
                denom = sigma'*w;
                x = x + step*(osig.*s); 
                if (num >= 0) |  (k >= nbr)  % B.POINT IS A MIN OR (k >= nbr) %
                   alpha=oldalf+tau*step;
                   x = x - (1-tau)*step*(osig.*s);
                   sigma(beta) = sig;
                   done=2;
                k = k
                else
                   oldalf=alpha;
                end
             end
       end
