function [it,fc,val,x]=main(func,grad,hess,tol,itbound,x,u,l,tsub)
%  
%  A trust region method for minimizing a nonlinear function subject to bounds
%
%  tol - tolerance for solving problem
%  itbound - maximum number of iterations
%  March 10, 93, Yuying Li
%
mu = 0.25;
n = length(x);
tsub = min(tsub,n);

val = inf; fc =0; gc=0;
[x]=Perturb(x,l,u);

newval = eval( [func,'(x)' ] );
g = eval([grad,'(x)']); H = eval([hess,'(x)']);
H = sparse(H);
c=g-H*x;
gc=gc+1; fc=fc+1; p=symmmd(H);

[v,dv]=DefineV(g,x,l,u);
sqrtv = sqrt(abs(v));
hatg = sqrtv.*g; posdef =0;

delta0 = 0.5*(sqrt(sum(min(x-l,u-x)))+1);
deltaU = delta0; deltaL=1;
delta0 = min(0.1*norm(g),delta0);
delta = delta0; qval=-1;

%
%  Main loop
%
flop1 = flops;
for it = 1:itbound
    vdelta(it)=delta;
    diff = abs(val-newval);
    vval(it) = val; VGnorm = norm(v.*g,inf);
    vvg(it) = VGnorm;
    if ( posdef>0 &  qval > -tol)
         break;
    end
    if( qval > 0 ) break;end;
    val = newval;
%
%   form the quadratic model hat{psi}_k(w)
%
    r = abs(min(x-l,u-x));
    D =sparse(1:n,1:n,sqrtv);
    DS =sparse(1:n,1:n,ones(n,1));
    M = D*H*D + sparse(1:n,1:n,(g.*dv));
    M=(M+M')/2; 
    deg = min(r+abs(g));
    if ( deg < 10^(-3)) deg = deg,end;
%
%   Solve a subspace trust region problem
%
    theta = max(0.95,1-VGnorm);
    [ss,qval,posdef,rho_c]=Solve2D(x,l,u,M,sqrtv,p,hatg,delta,tsub,theta);
    vrho_c(it) = rho_c;
    vpos(it) = posdef;
    s = sqrtv.*ss;
    newval = eval( [func,'(x+s)' ] );
    fc = fc+1;
    sTCs = ss'*(g.*dv.*ss);
    rho_f=(newval+0.5*sTCs-val)/qval;
    vrho_f(it) = rho_f;
%
%   Determine the trust region size for the next iteration
%
    [delta]=UpdateDelta(rho_f,rho_c,delta,ss,deltaL,deltaU);
    if( rho_f > mu )
        x=x+s;
%       [x]=Perturb_eps(x,l,u);
%       newval = eval([func,'(x)']);
        g = eval([grad,'(x)']); H = eval([hess,'(x)']);
        H = sparse(H);
        gc=gc+1;
        [v,dv]=DefineV(g,x,l,u);
        sqrtv = sqrt(abs(v));
        hatg = sqrtv.*g;
    else
        disp('fail');
        newval = val;
    end;
end
Tflop = flops-flop1;
val = vval(it);
Number_of_Iterations =it
Degeneracy = min(abs(v)+abs(g));
if Degeneracy < 10^(-5)
   Degeneracy = Degeneracy
end;
posdef = posdef
VGnorm = VGnorm
%save /usr/fsys/amma/a/yuying/matlab/nonlinear/Trust/Ucsln x
if ( it >= itbound ) keyboard;end;
clear B
return
