function [s,val,posdef,key,count,lambda] = trust(g,H,delta)
%
% SOLVE THE TRUST REGION PROBLEM:
%
%    min{g^Ts + 1/2 s^THs: ||s|| <= delta}
%
% THE FULL EIGEN-DECOMPOSITION IS COMPUTED. THE
% SECULAR EQUATION, 1/delta - 1/(||s||) = 0, is used.
%
% ON RETURN:
%
%  val: THE VALUE OF g^Ts + 1/2 s^THs.
%  posdef: posdef = 1 IF H IS POS. DEFINITE, 0 OTHERWISE.
%  key: A FLAG.
%  count: THE NUMBER OF EVALUATIONS OF THE SECULAR EQUATION. 
%  lambda: THE OPTIMAL LAGRANGE MULTIPLIER 
%
  tol = 10^(-12); tol2 = 10^(-8); key = 0; itbnd = 50;
  n = length(g); coeff(1:n,1) = zeros(n,1);
  [V,D] = eig(H);
  eigval = diag(D); [mineig,jmin] = min(eigval);
  alpha = -V'*g; sig = sign(alpha(jmin)) + (alpha(jmin)==0);
% save eigval eigval; save alpha alpha; save delta delta;
%
% POSITIVE DEFINITE CASE
  if mineig > 0
     coeff = alpha ./ eigval; lambda = 0;
     s = V*coeff; posdef = 1; nrms = norm(s);
     if nrms <= 1.2*delta
        key = 1;
     else 
        laminit = 0;
     end
  else
     laminit = -mineig; posdef = 0;
  end
%
% INDEFINITE CASE
  if key == 0
     if sec(laminit,eigval,alpha,delta) > 0
        [b,c,count] = Rfzero('sec',laminit,itbnd,eigval,alpha,delta,tol);
        vval = abs(sec(b,eigval,alpha,delta));
        if abs(sec(b,eigval,alpha,delta)) <= tol2
           lambda = b; key = 2;
           lam = lambda*ones(n,1);
           w = eigval + lam;
           arg1 = (w==0) & (alpha == 0); arg2 = (w==0) & (alpha ~= 0);
           coeff(w ~=0) = alpha(w ~=0) ./ w(w~=0);
           coeff(arg1) = zeros(length(arg1(arg1>0)),1);
           coeff(arg2) = Inf *ones(length(arg2(arg2>0)),1);
           coeff(coeff==NaN)=zeros(length(coeff(coeff==NaN)),1);
           s = V*coeff; nrms = norm(s);
           if (nrms > 1.2*delta) | (nrms < .8*delta)
              key = 5; lambda = -mineig;
            end
        else
           lambda = -mineig; key = 3;
         end
      else
         lambda = -mineig; key = 4;
      end
      lam = lambda*ones(n,1);
      if (key > 2) 
         arg = abs(eigval + lam) < 10 * eps * max(abs(eigval),ones(n,1));
         alpha(arg) = zeros(length(arg(arg>0)),1);
      end
      w = eigval + lam;
      arg1 = (w==0) & (alpha == 0); arg2 = (w==0) & (alpha ~= 0);
      coeff(w~=0) = alpha(w~=0) ./ w(w~=0);
      coeff(arg1) = zeros(length(arg1(arg1>0)),1);
      coeff(arg2) = Inf *ones(length(arg2(arg2>0)),1);
      coeff(coeff==NaN)=zeros(length(coeff(coeff==NaN)),1);
      coeff(coeff==NaN)=zeros(length(coeff(coeff==NaN)),1);
      s = V*coeff; nrms = norm(s);
      if (key > 2) & (nrms < .8*delta)
         beta = sqrt(delta^2 - nrms^2);
         s = s + beta*sig*V(:,jmin);
       end
       if (key > 2) & (nrms > 1.2*delta)
%         save alpha alpha
          [b,c,count] = Rfzero('sec',laminit,itbnd,eigval,alpha,delta,tol);
          lambda = b; lam = lambda*(ones(n,1));
          w = eigval + lam;
          arg1 = (w==0) & (alpha == 0); arg2 = (w==0) & (alpha ~= 0);
          coeff(w~=0) = alpha(w~=0) ./ w(w~=0);
          coeff(arg1) = zeros(length(arg1(arg1>0)),1);
          coeff(arg2) = Inf *ones(length(arg2(arg2>0)),1);
          coeff(coeff==NaN)=zeros(length(coeff(coeff==NaN)),1);
          s = V*coeff; nrms = norm(s);
        end
  end
% if nrms > 1.2*delta, s = s*(delta/nrms); end
  val = g'*s + (.5*s)'*(H*s);
%
% LAST LINE OF trust
