function[v,posdef] = newtorneg(M,p,g)
%
%  THIS ROUTINE DETERMINES THE NEWTON STEP, Mv = -g,
%  IF M IS FOUND TO BE POSITIVE DEFINITE. OTHERWISE,
%  A DIRECTION OF NON-POSITIVE CURVATURE IS RETURNED IN v.
%
%  INPUT VARIABLES (UNCHANGED):
%   M: A SYMMETRIC MATRIX STORED IN SPARSE FORMAT.
%   p: A VECTOR IDENTIFYING A PERMUTATION FOR M.
%   g: THE RIGHT-HAND-SIDE
%   
%  OUTPUT VARIABLES:
%   v: EITHER THE NEWTON DIRECTION, Mv = -g, OR 
%      A DIRECTION OF NON-POSITIVE CURVATURE, v'Mv <= 0.
%   posdef: posdef = 1 INDICATES M IS POS. DEF; posdef = 0 OTHERWISE.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%      Tom Coleman, Cornell University.  December 5, 1991.    %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

n = length(g); v = zeros(n,1); 
NEWT = 1;
[R,info] = chol(M(p,p));

if info == 0
   temp = R'\g(p); v(p,1) = -R\temp;
else
   if info == 1
      v(1) = 1;
   else
      j = info-1;
      [t1,t2]=size(R);
      if ( j+1>t2| j==0|j >t1 )
         disp('fail to compute a negative curvature direction from chol');
         v= zeros(n,1); keyboard;
      else
         v(1:j,1) = -R(1:j,1:j)\R(1:j,j+1); v(j+1,1) = 1;
      end
   end
   v(p) = v;
end

vg = v'*g;
sgn = sign(vg) + (vg == 0);
v = -sgn*v;
if info == 0, posdef = 1; else, posdef = 0; end

clear R;
