function [call,put] = blsfddelta(SO,X,r,T,sig,q,M,N)
%
%BLSPRICEFD Black-Scholes put and call pricing for european options using finite difference
%	solution of B-S Partial differential equation. Note that this function
%	returns an approximate solution unlike the analytical solution (BLSPRICE)
%
%       [CALL,PUT] = BLSPRICEFD(SO,X,R,T,SIG,Q,M,N) returns the value of
%       call and put options using the Black-Scholes pricing formula.  SO is
%       the current asset price, X is the exercise price, R is the risk-free
%       interest rate, T is the time to maturity of the option in years, SIG
%       is the standard deviation of the annualized continuously compounded
%       rate of return of the asset (also known as volatility), and Q is the
%       dividend rate of the asset.  The default Q is 0. M denotes the number
%       of discretization points in the stock price domain, and N denotes 
%	the number of discretization points in time domain used for the PDE solution.
%       Try increasing either of M or N to achieve greater efficiency. 
%       
%       For example, the current price an asset is $100, the exercise price 
%       of the option is $95, the risk-free interest rate is 10%, the time 
%       to maturity of the option is .25 years, and the standard deviation of 
%       of the asset is 50%.  Using this data,  
% 
%       call = blspricefd(100,95,.1,.25,.5,0) 
% 
%       returns a call option price of $13.64. 
%        
%       Reference: John C Hull : Options, Futures and other derivatives, Chap 15.
% 
%       See also BLSPRICEFDAM, BLSPRICE, BLSFDGAMMA


if nargin < 7 M =20; end
if nargin < 8 N =20; end


[call,put]=blspricefd(SO,X,deriv(r,ones(size(r))),T,sig,q,M,N);

call=getydot(call);
put=getydot(put);
