function [A,c,a,A1,c1,a1] = compA(sigma,r,q,T,M,N)
% compA  Sets up the finite difference matrices for crank-nicholson solution
%
%       Reference: John C Hull : Options, Futures and other derivatives, Chap 15.

dt = T/(M-1);
x = linspace(1,N-2,N-2); x=x(:);
sigma=sigma(2:N-1);

%Implicit constants
a=0.5*(r-q)*dt*x-0.5*sigma.*sigma.*x.*x*dt;
b= 2+sigma.*sigma.*x.*x*dt+2*r*dt;
c= -0.5*(r-q)*dt*x-0.5*sigma.*sigma.*x.*x*dt;


%Explicit constants
a1= -a;
b1= 2-sigma.*sigma.*x.*x*dt;
c1= -c;

A=spdiags([[a(2:N-2);0] b [0; c(1:N-3)]],-1:1,N-2,N-2);
A1=spdiags([[a1(2:N-2);0] b1 [0; c1(1:N-3)]],-1:1,N-2,N-2);

