function A = compA(sigma,r,q,T,M,N)
% compA  Sets up the finite difference matrix for implicit solution
%        A is a tridiagonal matrix.
%
%       Reference: John C Hull : Options, Futures and other derivatives, Chap 15.

dt = T/(M-1);
x = linspace(1,N-2,N-2)';
sigma=sigma(2:N-1);
temp1=sigma.*sigma.*x.*x*dt;
temp2=(r-q)*dt*x;

a= 0.5*(temp2-temp1);
b= 1+temp1+r*dt;
c= -0.5*(temp1+temp2);
A=spdiags([[a(2:N-2);-1;0] [b;1] [0; c(1:N-2)]],-1:1,N-1,N-1);
