%
% Example of european call option where sigma =sigma(S)
%
%Initialise
global OBSERVED

S0=590; r =.06;
q = 0.0262;
valconds ='variational';

T=[.175;.425; .695;.94;1;1.5;2];
K=S0*[.85;.9;.95;1;1.05;1.1;1.15;1.2;1.3;1.4];

implsig=[.190 .168 .133 .113 .102 .097 .120 .142 .169 .200
         .177 .155 .138 .125 .109 .103 .100 .114 .130 .150
         .172 .157 .144 .133 .118 .104 .100 .101 .108 .124
         .171 .159 .149 .137 .127 .113 .106 .103 .100 .110
         .171 .159 .150 .138 .128 .115 .107 .103 .099 .108
         .169 .160 .151 .142 .133 .124 .119 .113 .107 .102
         .169 .161 .153 .145 .137 .130 .126 .119 .115 .111
         .168 .161 .155 .149 .143 .137 .133 .128 .124 .123
         .168 .162 .157 .152 .148 .143 .139 .135 .130 .128
         .168 .164 .159 .154 .151 .148 .144 .140 .136 .132];

implsig=implsig(1:7,:);

Val=zeros(size(implsig));

for i=1:length(T)
    for j=1:length(K)
        Val(i,j)=blsprice(S0,K(j),r,T(i),implsig(i,j),q);
    end
end
Val = Val';
OBSERVED = Val(:);


N=65;                % number of stock steps
numtimesteps=25;      % number of time steps

uknotT = [0;.175;.425; .695;.94;1;1.5;2];
uknotS = S0*[.85;.9;.95;1;1.05;1.1;1.15;1.2;1.3;1.4];
knotT = [uknotT];
mn = 1;
knotS = [0; 0.25*S0; 0.5*S0; 0.75*S0; uknotS; 1.55*S0; 1.75*S0; 2*S0];

numknotS=length(knotS);
numknotT=length(knotT);
TT = kron(T,ones(length(K),1));
KK = kron(ones(length(T),1),K);

Extra.S0=S0;
Extra.r=r;
Extra.q=q;
Extra.T=TT;
Extra.K=KK;
Extra.numtimesteps=numtimesteps;
Extra.N=N;
Extra.knotT=knotT;
Extra.knotS=knotS;
numUT = length(uknotT);
numUS = length(uknotS);
Extra.Index= [kron(ones(numUT,1),[1;2;3;4;(4+numUS+1:4+numUS+3)']),...
              kron((1:numUT)',ones(7,1))];
Extra.consts= kron(ones(numUT,1),[0.14;0.14;0.14;0.14;0.14;0.14;0.14]);
numconsts=size(Extra.Index,1)


fun = 'funopt2dnew';

XX =ppual(pp,{knotS, knotT});
mesh(knotT,knotS,XX);
title('Initial Volatility without Boundary');

ystart =ppual(pp,{uknotS, uknotT});

y=ystart(:);

I=Extra.Index(:,1);
J=Extra.Index(:,2);
consts=Extra.consts;
count=1;
count2=1;
count3=1;
yval=y;

for j=1:length(knotT)
for i=1:length(knotS)

if (count3 <= length(consts)) & I(count3)==i & J(count3)==j
yval(count)=consts(count3);
count3=count3+1;
else
yval(count)=y(count2);
count2=count2+1;
end
count=count+1;
end
end

initSigma= reshape(yval,length(knotS),length(knotT));

%mesh(knotT,knotS,initSigma);

pp=csape({knotS,knotT},initSigma,valconds);

gridS = linspace(0,2*S0,N)';
gridT = linspace(0,2,numtimesteps)';

initGridSigma =ppual(pp,{gridS, gridT});
mesh(gridT,gridS,initGridSigma);
title('Initial Implied Sigma at Grid Points');

drawnow;

ystart=y;


n=numknotS*numknotT-numconsts


%%%%%
% DERIVATIVE of spline
%%%%%
global Jspline
Jspline=computeJspline(yval,Extra,valconds);



tol = 0.5*10^(-4); itbound = 50;

l = -0.5*ones(n,1);
u = 0.5*ones(n,1);

xx= funopt2dnew(ystart,Extra)

[it,gc,val,y]= mainFull(fun,Extra,tol,itbound,ystart,u,l);

I=Extra.Index(:,1);
J=Extra.Index(:,2);
consts=Extra.consts;
count=1;
count2=1;
count3=1;
yval=y;

for j=1:length(knotT)
for i=1:length(knotS)

if (count3 <= length(consts)) & I(count3)==i & J(count3)==j
yval(count)=consts(count3);
count3=count3+1;
else
yval(count)=y(count2);
count2=count2+1;
end
count=count+1;
end
end

p=length(knotS);
q=length(knotT);
knotSigma=reshape(yval,p,q);
pp=csape({knotS,knotT},knotSigma,valconds);

figure
subS = linspace(S0*0.8,S0*1.45,60);
subT = linspace(0,2,25);
subSigma=ppual(pp,{subS, subT});

mesh(subT,subS,subSigma)
title(sprintf('Reconstructed Volatility in the SubRegion'));


figure

gridS = linspace(0,2*S0,N)';
gridT = linspace(0,2,numtimesteps)';

optSigma=ppual(pp,{gridS, gridT});

optSigma = sqrt(optSigma.^2);

mesh(gridT,gridS,optSigma)

title(sprintf('Reconstructed Volatility with %2.0f Observations #Jeval=%2.0f',m,gc));

Extra.knotT=gridT;
Extra.knotS=gridS;
Extra.Index= [];
Extra.consts=[];

rlError = funopt2dnew(optSigma(:),Extra)

save SP500 r q S0 T K implsig N numtimesteps gridT gridS optSigma TT KK

