%
% Example of european call option where sigma =sigma(S)
%
echo off
%Initialise
global OBSERVED

S0=590; r =.06;
q = 0.0262;
valconds ='variational';

T=[.175;.425; .695;.94;1;1.5;2];
K=S0*[.85;.9;.95;1;1.05;1.1;1.15;1.2;1.3;1.4];

implsig=[.190 .168 .133 .113 .102 .097 .120 .142 .169 .200
         .177 .155 .138 .125 .109 .103 .100 .114 .130 .150
         .172 .157 .144 .133 .118 .104 .100 .101 .108 .124
         .171 .159 .149 .137 .127 .113 .106 .103 .100 .110
         .171 .159 .150 .138 .128 .115 .107 .103 .099 .108
         .169 .160 .151 .142 .133 .124 .119 .113 .107 .102
         .169 .161 .153 .145 .137 .130 .126 .119 .115 .111
         .168 .161 .155 .149 .143 .137 .133 .128 .124 .123
         .168 .162 .157 .152 .148 .143 .139 .135 .130 .128
         .168 .164 .159 .154 .151 .148 .144 .140 .136 .132];

implsig=implsig(1:7,:);

Val=zeros(size(implsig));

for i=1:length(T)
    for j=1:length(K)
        Val(i,j)=blsprice(S0,K(j),r,T(i),implsig(i,j),q);
    end
end

figure;

mesh(log(K),T,implsig)
title(sprintf('Implied Volatility'));
drawnow;

m = length(T)*length(K);

TT = kron(T,ones(length(K),1));
KK = kron(ones(length(T),1),K); 
Val = Val';
OBSERVED = Val(:);

N=30;                % number of stock steps
numtimesteps=10;      % number of time steps

Extra=[S0;r;q;numtimesteps;TT;KK];

uknotT = [0;.175;.425; .695;.94;1;1.5;2];
uknotS = S0*[.85;.9;.95;1;1.05;1.1;1.15;1.2;1.3;1.4];
knotT = [uknotT];
mn = 1;
knotS = [0; 0.25*S0; 0.5*S0; 0.75*S0; uknotS; 1.55*S0; 1.75*S0; 2*S0];

numknotS=length(knotS);
numknotT=length(knotT);

Extra.S0=S0;
Extra.r=r;
Extra.q=q;
Extra.T=TT;
Extra.K=KK;
Extra.numtimesteps=numtimesteps;
Extra.N=N;
Extra.knotT=knotT;
Extra.knotS=knotS;
Extra.Index=[];
Extra.consts=[];

fun = 'funopt2dnew';

pp=csape({K,T},implsig',valconds);

ystart =ppual(pp,{knotS, knotT});



initSigma= ystart;

pp=csape({knotS,knotT},initSigma,valconds);

gridS = linspace(0,2*S0,N)';
gridT = linspace(0,2,numtimesteps)';

subS = linspace(S0*0.8,S0*1.45,60);
subT = linspace(0,2,25);
subSigma=ppual(pp,{subS, subT});

mesh(subT,subS,subSigma)
title(sprintf('Volatility in the SubRegion'));


Extra.T=repmat(gridT',N,1);
Extra.T=Extra.T(:);
Extra.K=repmat(gridS,1,numtimesteps);
Extra.K=Extra.K(:);
Vals=optionfunspline2d(ystart,Extra);
Vals=reshape(Vals,N,numtimesteps);

figure;
mesh(gridT,gridS,Vals);
title('Option price at Grid Points');
drawnow;


