function s= derivspj(a,V)
%
% DERIVSPJ  --  Construct DERIVSPJ Objects
%
%   S=deriv(a) constructs the object S, of class deriv from a.
%   If the input argument is a numerical scalar or matrix, the
%   result is a deriv object having the same value and
%   derivative field set to 0.
%
%   If input is a deriv object, it is returned as it is.
%
%   S=deriv(a,V) sets the derivative field of the output to
%   be equal to V.
%
%   Also see getval, getydot, derivtape, derivspj, derivsph
global globp;
if nargin==2 globp=size(V,2); end

if nargin==0
    s.val=0;
    s.derivspj=sparse(1,globp);
    s=class(s,'derivspj');
elseif nargin==1
    
    if isempty(a)
        s.val=[];
        s.derivspj=[];
        s=class(s,'derivspj');
    elseif isa(a,'derivspj')
        s=a;
        
    else
        s.val=a;
        if isstruct(a) error('Wrong use of derivspj class'); end
        [m,n]=size(getval(a));
        if ((m==1) &(n==1))
            s.derivspj=sparse(1,globp);
        elseif (m==1)
            s.derivspj=sparse(n,globp);
        elseif (n==1)
            s.derivspj=sparse(m,globp);
        else
            %	if globp > 1 
            s.derivspj=cell(globp,1);
            for i=1:globp s.derivspj{i}=sparse(m,n); end
            %	else
            %		s.derivspj=sparse(m,n);
            %	end
        end
        s=class(s,'derivspj');
    end
    
    
else
    s=derivspj(a);
    s.derivspj=V; 
end
