function [value, grad] = cragglev(x,dummy)
%
% Evaluates the CRAGGLEVY test problem, described in ??.
%
% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Input:
%       x - the current point (column vector).
%
% Output:
%        value - (scalar) objective function value at x.
%         grad - gradient of the objective fcn. (column vector) evaluated at x.
%         H    -  Hessian (sparse symmetric matrix).
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Evaluate the function.
        n=length(x); y=zeros(n,1);
        if rem(n,4)
          error('  For this problem, n must be a multiple of 4.')
        end
        i=1:4:(n-3);
        y(i,1) = (exp(x(i,1))-x(i+1,1)).^4+100*(x(i+1,1)-x(i+2,1)).^6 ...
               +(tan(x(i+2,1)-x(i+3,1))).^4+x(i,1).^8+(x(i+3,1)-1).^2;
        value=sum(y);
%
% Evaluate the gradient.
  if nargout > 1
    g = zeros(n,1);
    i=1:4:(n-3);
    g(i,1)=4*((exp(x(i,1))-x(i+1,1)).^3).*exp(x(i,1))+...
           8*x(i,1).^7;
    g(i+1,1)=-4*(exp(x(i,1))-x(i+1,1)).^3+...
           600*(x(i+1,1)-x(i+2,1)).^5;
    g(i+2,1)=-600*(x(i+1,1)-x(i+2,1)).^5+...
             4*(1+(tan(x(i+2,1)-x(i+3,1))).^2).*(tan(x(i+2,1)-x(i+3,1))).^3;
    g(i+3,1)= 2*(x(i+3,1)-1)-...
             4*(1+(tan(x(i+2,1)-x(i+3,1))).^2).*(tan(x(i+2,1)-x(i+3,1))).^3;
    grad = g;
  end
%
