function[fvec,A] = nlsf2(x,DM)
%NLSF2	Molecular conformation function (1-d)
%
%	[fvec,A] = nlsf2(x,DM);
%
%	Evaluate the distance vector) function (and possibly Jacobian) for
%	the molecule problem (1-dimensional)
%
%       Input:
%       _____
%       x - The current point (column vector), i.e., positions of the
%	    atoms (on real line).
%	DM - Sparse `distance matrix'.
%
%       Output:
%       ______
%       fvec - The (vector) function value at x. Component k of
%              fvec corrsponds to the kth (i,j)-pair
%              in DM witrh values (x(i)-x(j))^2 -dist^2.
%       A    - The (sparse) rectangular Jacobian matrix.
%
%       ******************************************************************
%       *              Thomas F. Coleman, 7-1-96                         *
%       *              Copyright (c) 1996 by The Mathworks, Inc.         *
%       ******************************************************************
%
%   Evaluate the distance function.
    ndist = []; n = [];
    [I,J,dist] = find(DM);
    ndist = length(dist); n = length(x);
    term1 = x(I) -x(J);
    term2 = term1.*term1;
    fvec = term2 - dist.*dist;
    fvec(ndist+1,1) = x(1)-1;
%
%   Determine the the Jacobian (update edge by edge)
    if nargout > 1
       A = sparse(1:ndist,1:n);
       for k = 1:ndist
           i = I(k); j = J(k);
           A(k,i) = 2*term1(k); A(k,j) = -2*term1(k);
        end
        A(ndist+1,1) = 1;
    end 
