function [f,g,extH,ns,H]=evalHExt(class,fun,x,fdata,EHPI,verb)
%
%evalHExt
%
%       Compute Extended Hessian, Newton step.
%
%       f=evalHExt(class,fun,x)  
%       [f,g]=evalHExt(class,fun,x)  
%       [f,g,extH]=evalHExt(class,fun,x) 
%       [f,g,extH,ns]=evalHExt(class,fun,x)
%       [f,g,extH,ns,H]=evalHExt(class,fun,x)
%
%       Please refer to ADMIT-2 users manual for complete reference.
%
%       EHPI argument can be used to efficient execution. 
%       SEE GetExtHPI
%
%       ALSO SEE configureExtfun, evalJExt, ExtJV, ExtWJ, ExtHV, ExtVHV
%
%
%       ******************************************************************
%       *              ADMIT-2 Project   1996-97                         *
%       *              Copyright (c) 1996 Cornell University.            *
%       ******************************************************************

n=length(x);

% Initialize
 
if nargin < 4 fdata=[]; end
if nargin < 5 EHPI=[]; end
if nargin < 6 verb=[]; end
if isempty(verb) verb=0; end
 
if isempty(fdata) [fun,fdata]=MakeExtFdata(class,fun,n); end
  
 
if nargout == 1
        f=funcvalExt(class,fun,x,fdata);
elseif nargout == 2
	[f,g]=funcvalExt(class,fun,x,fdata);
else

% Make the function data
 
if isempty(EHPI)
        EHPI=GetExtHPI(class,fun,n,fdata);
end
if class=='GF'
	[f,g,extH]=evalHgen(fun,fdata,x,EHPI);
end
if class=='GC'
        [f,g,extH]=evalHGC(fun,fdata,x,EHPI);
end
if class=='GP'
        [f,g,extH]=evalHGP(fun,fdata,x,EHPI);
end
if class=='DO'
        [f,g,extH]=evalHDO(fun,fdata,x,EHPI);
end
if class=='GI'
	impl=fdata.impl;
	if impl==0
	if (fdata.kind==1)
        [f,g,extH]=evalHDI1(fun,fdata,x,EHPI);
	elseif (fdata.kind==2)
        [f,g,extH]=evalHDI2(fun,fdata,x,EHPI);
	else
        [f,g,extH]=evalHDI3(fun,fdata,x,EHPI);
	end

	else

        if (fdata.kind==1)
        [f,g,extH]=evalHII1(fun,fdata,x,EHPI);
        elseif (fdata.kind==2)
        [f,g,extH]=evalHII2(fun,fdata,x,EHPI);
        else
        [f,g,extH]=evalHII3(fun,fdata,x,EHPI);
        end
	end
end

[nrows,ncols]=size(extH);
 
if nargout >= 4
%%%% Exploit symmetry; do some tests here
	rhs=[zeros(nrows-n,1);-g];
        ns=extH\rhs;
        ns=ns(1:n);
end
 
 
if nargout ==5
        B=extH(nrows-n+1:nrows,1:n);
	if nrows==n 
		H=B;
	else
        	A=extH(1:nrows-n,1:n);
        	L=extH(1:nrows-n,n+1:nrows);
       		M=extH(nrows-n+1:nrows,n+1:nrows);
        	H=B-M*(L\A);
	end
end
 
        if verb >= 1
                if class=='GP'
                disp(sprintf('Number of Partially separable components = %d',fdata(1)));
                end
                if class=='GC'
                disp(sprintf('Number of composite functions = %d',fdata(1)));
                end
                if class=='GF'
                disp(sprintf('Number of functions in general structured program = %d',fdata(1)));
                end
                if class=='GI'
 disp(sprintf('The time interval =[%f,%f], step size =%f',fdata(1),fdata(2),fdata(3)));
                end
                disp(sprintf('Problem Size = %d -> %d',n,1));
        end
        if verb==2
                if nargout==5
                        subplot(2,1,1)
                        spy(extH)
                        title 'Sparsity Structure of Extended Hessian Matrix'
                        subplot(2,1,2)
                        spy(H)
                        title 'Sparsity Structure of True Hessian Matrix'
                else
                        spy(extH)
                        title 'Sparsity Structure of Extended Hessian Matrix'
                end
        end

end
